/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "db/DBFactory.h"
#include "db/Account.h"
#include "db/Guard.h"
#include "db/Pref.h"
#include "db/Price.h"
#include "db/Portfolio.h"
#include "db/Security.h"
#include "db/Set.h"
#include "db/SetItem.h"
#include "Enums.h"
#include "Progress.h"
#include <MHCSVFile.h>
#include <MHDate.h>
#include <MHDebug.h>
#include <MHVector.h>

char        DBFactory::aaPathSep                    = '\0';
int         DBFactory::aaCacheSize                  = DBFactory::CACHE_SIZE;
int         DBFactory::aaCacheTimeout               = DBFactory::CACHE_TIMEOUT;
int         DBFactory::aaPriceAccess[CACHE_SIZE];
MHString*   DBFactory::aaLogName                    = 0;
MHString*   DBFactory::aaPath                       = 0;
MHString*   DBFactory::aaWorkSpace                  = 0;
MHVector*   DBFactory::aaAccount                    = 0;
MHVector*   DBFactory::aaClosed[CACHE_SIZE];
MHVector*   DBFactory::aaGuard                      = 0;
MHVector*   DBFactory::aaPref                       = 0;
MHVector*   DBFactory::aaPrice[CACHE_SIZE];
MHVector*   DBFactory::aaSecurity                   = 0;
MHVector*   DBFactory::aaSet                        = 0;
Security*   DBFactory::aaPriceName[CACHE_SIZE];
#define SLEEPING //MHDate::SLEEPING (1);



/**
*
*/
void DBFactory::Clear () {
    if (DBFactory::aaAccount) DBFactory::aaAccount->Erase();
    if (DBFactory::aaPref) DBFactory::aaPref->Erase();
    if (DBFactory::aaGuard) DBFactory::aaGuard->Erase();
    if (DBFactory::aaSecurity) DBFactory::aaSecurity->Erase();
    if (DBFactory::aaSet) DBFactory::aaSet->Erase();
    ClearPrice ();
}



/**
*
*/
void DBFactory::ClearPrice () {
    for (int f = 0; f < CACHE_SIZE; f++) {
        aaPrice[f]->Erase();
        aaClosed[f]->Erase();
        delete aaPriceName[f];
        aaPriceName[f]   = 0;
        aaPriceAccess[f] = 0;
    }
}



/**
*  Export all database or parts of to xtrader import text file
*/
void DBFactory::Export (const char* fileName, bool account, bool transactions, bool set, bool security, bool price, bool guard, bool pref) {
    MHString    name = DBFactory::Database();
    MHVector*   ve;
    MHVector    ve2;
    int         f;
    FILE*       fs = fopen (fileName, "w");

    if (!fs) throw (STRING(ERROR_SAVE));
    DBFactory::Clear ();

    //---------------------------------
    ve = DBFactory::LoadSecurity();
    xtrader_reset (7 + ve->Size());
    if (security)
        Security::Export (ve, fs);
    xtrader_add(1);
    if (xtrader_abort()) return;
    SLEEPING

    //---------------------------------
    if (price) {
        for (f = 0; f < ve->Size(); f++) {
            Security* sec = (Security*)(*ve)[f];
            Price::LoadAll(sec, &ve2);
            Price::Export (sec, &ve2, fs);
            ve2.Erase();
            xtrader_add(1);
            SLEEPING

        }
    }
    else
        xtrader_add(ve->Size());

    //---------------------------------
    if (pref) {
        ve = DBFactory::LoadPref();
        Pref::Export (ve, fs);
    }
    xtrader_add(1);
    if (xtrader_abort()) return;
    SLEEPING

    //---------------------------------
    ve = DBFactory::LoadAccount();
    if (account)
        Account::Export (ve, fs);
    xtrader_add(1);
    if (xtrader_abort()) return;
    SLEEPING

    //---------------------------------
    if (transactions) {
        for (f = 0; f < ve->Size(); f++) {
            Account* acc = (Account*)(*ve)[f];
            Portfolio::Load (acc, &ve2);
            Portfolio::Export (acc, &ve2, fs);
            ve2.Erase ();
        }
    }
    ve->Erase ();
    xtrader_add(1);
    if (xtrader_abort()) return;
    SLEEPING

    //---------------------------------
    if (set) {
        ve = DBFactory::LoadSet();
        Set::Export (ve, fs);

        for (f = 0; f < ve->Size(); f++) {
            Set*     set = (Set*)(*ve)[f];
            MHVector*  vec = new MHVector ();

            DBFactory::LoadSetItem (set, vec);
            SetItem::Export (set, vec, fs);
            vec->Erase ();
            delete vec;
        }
    }
    xtrader_add(1);
    if (xtrader_abort()) return;
    SLEEPING

    //---------------------------------
    if (guard) {
        ve = DBFactory::LoadGuard();
        Guard::Export (ve, fs);
    }
    xtrader_add(1);

    fclose (fs);
}



/**
*  Get preference integer value
*  @param const char* - Key
*  @param int         - Legal min value
*  @param int         - Legal max value
*  @param int         - Default value if key is not found
*  @return int        - Value
*/
int DBFactory::GetPrefIntValue (const char* key, int min, int max, int def) {
    MHVector*  ve   = DBFactory::LoadPref ();
    Pref       p (key);
    Pref*      pref = (Pref*) ve->Find (&p, 0, 0, 0);

    if (pref) {
        int v = pref->GetValue()->ToInt();
        if (v >= min && v <= max)
            return v;
        else
            return def;
    }
    else
        return def;
}



/**
*  Get preference integer value
*  @param const char* - Key
*  @param int         - View number
*  @param int         - Legal min value
*  @param int         - Legal max value
*  @param int         - Default value if key is not found
*  @return int        - Value
*/
int DBFactory::GetPrefIntValue (const char* key, int view, int min, int max, int def) {
    static char key2[100];
    snprintf (key2, 99, "%s_%d", key, view);
    return DBFactory::GetPrefIntValue (key2, min, max, def);
}



/**
*  Get preference string value
*  @param const char*  - Key
*  @param const char*  - Default value of key is not found
*  @return const char* - Value
*/
const char* DBFactory::GetPrefStringValue (const char* key, const char* def) {
    MHVector*  ve   = DBFactory::LoadPref ();
    Pref       p (key);
    Pref*      pref = (Pref*) ve->Find (&p, 0, 0, 0);

    if (pref)
        return pref->GetValue()->Get();
    else
        return def;
}



/**
*  Get a unique id number.
*  Throws Exception at failure
*  @return int - The unique id
*/
int DBFactory::GetUniqueID () {
    MHString    name = DBFactory::Database ();
    name        += "id.db";
    int         id;


    try {
        try {
            MHCSVFile infile (name(), '\t');
            if (infile.ReadRow())
                id = infile.GetInt (0);
            else
                id = 0;
        }
        catch (const char*) {
            id = 0;
        }
        id++;

        MHCSVFile outfile;
        char      buffer[30];

        sprintf (buffer, "%d", id);
        outfile.WriteString (buffer);
        outfile.WriteFile (name());
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving id failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }

    return id;
}



/**
*
*/
MHVector* DBFactory::LoadAccount() {
    if (aaAccount->Size() > 0)
        return aaAccount;
    Account::Load (aaAccount);
    return aaAccount;
}



/**
*
*/
MHVector* DBFactory::LoadGuard() {
    if (aaGuard->Size() > 0)
        return aaGuard;
    Guard::Load (aaGuard);
    return aaGuard;
}



/**
*
*/
MHVector* DBFactory::LoadPref() {
    if (aaPref->Size() > 0)
        return aaPref;
    Pref::Load (aaPref);
    return aaPref;
}



/**
*
*/
void DBFactory::LoadPortfolio (Account* account, MHVector* out) {
    Portfolio::Load (account, out);
}



/**
*  Load price data either form cache or from disk
*  @param Security* - The security we want price data for
*/
MHVector* DBFactory::loadPrice(Security* security, bool closed) {
    DBG ("DBFactory::loadPrice")
    int f;
    int slot = -1;

    if (!security) return 0;

    //  Check if searched security does exist in cache
    Security* sec;
    for (f = 0; f < DBFactory::aaCacheSize; f++) {
        sec = (Security*) aaPriceName[f];

        if (sec && *sec == *security) {
            int diff = ((int) time(0)) - aaPriceAccess[f];
            if (diff > aaCacheTimeout) {
                slot = f;
                DBG_LOG1 ("Cache to old for %d", security->GetID());
                break;
            }
            DBG_LOG1 ("Cache found for %d", security->GetID());
            aaPriceAccess[f] = (int) time(0);

            if (closed)
                return aaClosed[f];
            else
                return aaPrice[f];
        }
    }

    //  Find available slot, if full erase oldest accessed slot
    if (slot == -1) {
        slot = 0;
        int oldest = aaPriceAccess[0];
        for (f = 1; f < DBFactory::aaCacheSize && oldest > 0; f++) {
            if (aaPriceAccess[f] < oldest) {
                oldest = aaPriceAccess[f];
                slot   = f;
            }
        }
    }

    DBG_LOG1 ("Cache NOT found for %d", security->GetID());
    delete aaPriceName[slot];
    Price::Load (security, aaPrice[slot], aaClosed[slot]);
    aaPriceName[slot]   = new Security(*security);
    aaPriceAccess[slot] = (int) time(0);

    if (closed)
        return aaClosed[slot];
    else
        return aaPrice[slot];
}



/**
*
*/
void DBFactory::LoadRawPrice (Security* security, MHVector* out) {
    Price::LoadAll (security, out);
}



/**
*  Load security objects from databse
*/
MHVector* DBFactory::LoadSecurity() {
    if (aaSecurity->Size() > 0)
        return aaSecurity;
    Security::Load (aaSecurity);
    return aaSecurity;
}



/**
*  Load set objects from databse
*/
MHVector* DBFactory::LoadSet() {
    if (aaSet->Size() > 0)
        return aaSet;
    Set::Load (aaSet);
    return aaSet;
}



/**
*
*/
void DBFactory::LoadSetItem (Set* set, MHVector* out) {
    SetItem::Load (set, out);
}



/**
*  Set integer value into application preference table
*  @param const char* - Key
*  @param int         - Legal min value
*  @param int         - Legam max value
*  @param int         - Default value if value is out of range
*  @param int         - Value
*/
void DBFactory::SetPrefIntValue (const char* key, int min, int max, int def, int val) {
    MHVector*   ve   = DBFactory::LoadPref ();
    Pref        p (key);
    Pref*       pref = (Pref*) ve->Find (&p, 0, 0, 0);

    val = (val >= min && val <= max) ? val : def;

    if (pref)
        pref->SetValue(val);
    else
        ve->InsertSorted (new Pref (key, val));
}



/**
*  Set preference integer value
*  @param const char* - Key
*  @param int         - View number
*  @param int         - Legal min value
*  @param int         - Legal max value
*  @param int         - Default value if value is out of range
*  @return int        - Value
*/
void DBFactory::SetPrefIntValue (const char* key, int view, int min, int max, int def, int val) {
    static char key2[100];
    snprintf (key2, 99, "%s_%d", key, view);
    DBFactory::SetPrefIntValue (key2, min, max, def, val);
}



/**
*  Set integer value into application preference table
*  @param const char* - Key
*  @param const char* - Value
*/
void DBFactory::SetPrefStringValue (const char* key, const char* val) {
    MHVector*   ve   = DBFactory::LoadPref ();
    Pref        p (key);
    Pref*       pref = (Pref*) ve->Find (&p, 0, 0, 0);

    if (pref)
        pref->SetValue(val);
    else
        ve->InsertSorted (new Pref (key, val));
}



/**
*  Set workspace file.
*  @param const char* - Filename
*/
void DBFactory::SetWorkSpace (const char* file) {
    if (aaWorkSpace) {
        delete aaWorkSpace;
        aaWorkSpace = 0;
        DBFactory::ClearPref ();
    }

    aaWorkSpace = new MHString (file);
}



/**
*
*/
void DBFactory::Start (const char* database) {
    aaAccount  = new MHVector ();
    aaGuard    = new MHVector ();
    aaPref     = new MHVector ();
    aaSecurity = new MHVector ();
    aaSet      = new MHVector ();

    for (int f = 0; f < CACHE_SIZE; f++) {
        aaPrice[f]       = new MHVector ();
        aaClosed[f]      = new MHVector ();
        aaPriceName[f]   = 0;
        aaPriceAccess[f] = 0;
    }

    aaPath = new MHString (database);
    if (aaPath->Size() > 0 && aaPath->GetChar(aaPath->Size() - 1) != '/' &&  aaPath->GetChar(aaPath->Size() - 1) != '\\')
        *aaPath += DBFactory::Slash ();

    aaLogName = new MHString (*aaPath);
    *aaLogName += "xtrader.log";
    if (DBFactory::GetPrefIntValue (STRING(PREF_COMPRESS), XTRADER_NO, XTRADER_YES, XTRADER_NO) == XTRADER_YES) MHCSVFile::Compress();

    //!!!Radera alla tmp filer i databaskatalogen via MHFileTransaction::ClearTmpFile (dir)
    //    eller mhutil
}



/**
*
*/
void DBFactory::Stop () {
    DBFactory::Clear();
    delete aaAccount;
    delete aaGuard;
    delete aaLogName;
    delete aaPath;
    delete aaPref;
    delete aaSecurity;
    delete aaSet;
    delete aaWorkSpace;

    for (int f = 0; f < CACHE_SIZE; f++) {
        delete aaPrice[f];
        delete aaClosed[f];
        delete aaPriceName[f];

        aaPrice[f]      = 0;
        aaClosed[f]     = 0;
        aaPriceName[f]  = 0;
    }

    aaPath      = 0;
    aaAccount   = 0;
    aaPref      = 0;
    aaGuard     = 0;
    aaSecurity  = 0;
    aaSet       = 0;
    aaWorkSpace = 0;
    aaLogName   = 0;
}



/**
*  Log message
*  @param char*  - First string
*  @param ...    - Additional data
*/
void DBFactory::WriteLog (const char *Message, ...) {
    MHDate          date;
    char            buffer[501];
    const char*     dateStr = date.Get (MHDate::FULL_DATE);
    FILE*           file;

    va_list args;
    va_start (args, Message);
    vsnprintf (buffer, 500, Message, args);
    va_end (args);

    if (MHFile::Size(aaLogName->Get()) > 500000) {
        MHString tmp;
        tmp = *aaLogName;
        tmp += ".old";
        unlink (tmp());
        if (rename (aaLogName->Get(), tmp())) return;
    }

    file = fopen (aaLogName->Get(), "a");
    if (file) {
        fwrite (dateStr, sizeof(char), strlen(dateStr), file);
        fwrite (" | ", sizeof(char), 3, file);
        fwrite (buffer, sizeof(char), strlen(buffer), file);
        fwrite ("\n", sizeof(char), 1, file);
        fclose (file);
    }
}
