/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/Guard.h>
#include <db/DBFactory.h>
#include <Resource.h>
#include <MHCSVFile.h>



/**
*  Create new object
*/
Guard::Guard () :
MH() {
    aSecurityID = 0;
    aType       = BUY_HIGH;
    aValue      = 0;
}



/**
*  Create new object
*  @param const Guard& - Object to copy
*/
Guard::Guard(const Guard& x) :
MH() {
    aSecurityID = x.aSecurityID;
    aType       = x.aType;
    aValue      = x.aValue;
}



/**
*  Create a new object
*  @param int         - Security ID
*  @param TYPE        - type
*  @param double      - watch price
*  @param const char* - Date to delete watch
*/
Guard::Guard (int sid, TYPE Type, double Value) :
MH() {
    SetSecurityID (sid);
    SetType (Type);
    SetValue (Value);
}



/**
*  Operator =
*  @param const Guard& - Object to copy
*/
Guard& Guard::operator=(const Guard& x) {
    if (this == &x) return *this;

    IsChanged (((Guard&)x).IsChanged());
    aSecurityID = x.aSecurityID;
    aType       = x.aType;
    aValue      = x.aValue;

    return *this;
}



/**
*
*/
int Guard::Compare (const MH* o, int type) {
    Guard* g = (Guard*) o;

    if (aSecurity.GetID() == 0)
        LoadSecurity();
    if (g->GetSecurity()->GetID() == 0)
        g->LoadSecurity();

    return aSecurity.Compare (g->GetSecurity(), Security::SORT_NAME);
}



/**
*  Export data to file
*/
void Guard::Export (MHVector* ve, FILE* file) {
    Guard*      i;
    Security*   s;
    char        buffer[200];

    for (int f = 0; f < ve->Size(); f++) {
        i = (Guard*) (*ve)[f];
        s = i->LoadSecurity();
        if (s) {
            snprintf (buffer, 199, "+g\t%s\t%d\t%f\n", s->GetCode()->Get(), i->aType, i->aValue);
            fwrite (buffer, sizeof(char), strlen (buffer), file);
        }
    }
}



/**
*  Return the trasnlated name of current signal type
*  @return const char* - Name of signal
*/
const char* Guard::GetTypeName () {
    switch (aType) {
    case SELL_LOW:
        return STRING(SIG_SELL_LOW);

    case BUY_LOW:
        return STRING(SIG_BUY_LOW);

    case SELL_HIGH:
        return STRING(SIG_SELL_HIGH);

    case BUY_HIGH:
        return STRING(SIG_BUY_HIGH);

    case BUY_MVG:
        return STRING(SIG_BUY_MVG);

    case SELL_MVG:
        return STRING(SIG_SELL_MVG);

    default:
        return "";
    }
}



/**
*  Load a vector of Guard objects.
*  @param MHVector& - Vector
*/
void Guard::Load (MHVector* ve) {
    MHString    name = DBFactory::Database();

    name += "guard.db";
    ve->Erase ();

    try {
        MHCSVFile file (name(), '\t');

        while (file.ReadRow()) {
            if (file.Cols() != 3)
                throw "Wrong number of columns in the file";
            ve->Push (new Guard (file.GetInt(0), (TYPE)file.GetInt(1), file.GetDouble(2)));
        }
    }
    catch (const char* x) {
        ve->Erase ();
        if (MHFile::Exist (name()) == true)
            DBFactory::WriteLog ("Loading guard failed file=%s, error=%s", name(), x);
    }
}



/**
* Load Security
*/
Security* Guard::LoadSecurity() {
    aSecurity.SetID(0);
    aSecurity.SetName("");

    MHVector* v = DBFactory::LoadSecurity();
    Security  s1(aSecurityID);
    Security* s2 = (Security*) v->Find (&s1, MHVector::FIND_EXACT, Security::SORT_ID, 0);
    if (s2)
        aSecurity = *s2;
    return &aSecurity;
}



/**
* Load Security
* @param MHVector& - Vector with Guard objects
*/
void Guard::LoadSecurities(MHVector* v) {
    for (int f = 0; f < v->Size(); f++)
        ((Guard*)(*v)[f])->LoadSecurity();
}



/**
*  Save a vector of Guard objects.
*  Throws exception at failure
*  @param MHVector& - Vector
*/
void Guard::Save (MHVector* ve) {
    Guard*      i;
    MHString    name = DBFactory::Database();

    name += "guard.db";
    ve->Sort();

    try {
        MHCSVFile file;
        char      buff[400];

        for (int f = 0; f < ve->Size(); f++) {
            i = (Guard*) (*ve)[f];
            sprintf (buff, "%d\t%d\t%.2f", i->GetSecurityID(), i->GetType(), i->GetValue());
            file.WriteString (buff);
        }
        file.WriteFile (name());
        MH::ClearChanged (ve);
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving guard failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }
}
