/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "db/Import.h"
#include "db/Account.h"
#include "db/DBFactory.h"
#include "db/Guard.h"
#include "db/Pref.h"
#include "db/Portfolio.h"
#include "db/Price.h"
#include "db/Set.h"
#include "db/SetItem.h"
#include "Progress.h"
#include <MHDate.h>
#include <MHCSVFile.h>
#include <MHFileTransaction.h>

#define CHAR(i)   (i < (v->Cols()) ? v->GetString(i) : "")
#define DOUBLE(i) (i < (v->Cols()) ? v->GetDouble(i) : 0.0)
#define INT(i)    (i < (v->Cols()) ? v->GetInt(i) : 0)



/**
*  Create import object and create log file
*  @param const char* - Name of file to convert
*/
Import::Import (const char* fileName) {
    aAccount    = 0;
    aAccounts   = 0;
    aErrors     = 0;
    aFileName   = fileName;
    aGuards     = 0;
    aIni        = 0;
    aPortfolios = 0;
    aRows       = 0;
    aSecurities = 0;
    aSecurity   = 0;
    aSet        = 0;
    aSetItems   = 0;
    aSets       = 0;

    aPortfolios.Resize (100);
    aPrices.Resize (500);

    DBFactory::WriteLog ("%s %s", STRING(IMPORT_STARTED), aFileName());
    DBFactory::Clear ();
    MHFileTransaction::Begin ();
}



/**
*  Destroy import object. Close log file
*/
Import::~Import () {
    DBFactory::WriteLog ("%s (%s=%d, %s=%d)", STRING(IMPORT_DONE), STRING(ROWS), aRows, STRING(ERROR_), aErrors);
}



/**
*  Save all unsaved data
*/
void Import::cleanup () {
    try {
        if (aIni && MH::IsVectorChanged (aIni) == true) Pref::Save (aIni);
        if (aSecurities && MH::IsVectorChanged (aSecurities) == true) Security::Save (aSecurities);
        if (aSecurity) Price::Save (aSecurity, &aPrices);
        if (aGuards && MH::IsVectorChanged (aGuards) == true) Guard::Save (aGuards);
        if (aSets && MH::IsVectorChanged (aSets) == true) Set::Save (aSets);
        if (aSet) SetItem::Save (aSet, &aSetItems);
        if (aAccounts && aAccounts->IsChanged () == true) Account::Save (aAccounts);
        if (aAccount) Portfolio::Save (aAccount, &aPortfolios);

        aPortfolios.Erase ();
        aPrices.Erase ();
        aSetItems.Erase ();
        DBFactory::Clear ();
        MHFileTransaction::Commit ();
    }
    catch (const char* message) {
        DBFactory::WriteLog ("%s (%s)", STRING(ERROR_IMPORT_ROLLBACK), message);
        aPortfolios.Erase ();
        aPrices.Erase ();
        aSetItems.Erase ();
        DBFactory::Clear ();
        MHFileTransaction::Rollback ();
        throw (message);
    }
}



/**
*  Import a text file into database. All errors are reported to xtrader.log in the database directory
*/
void Import::Read () {
    try {
        int critical = 0;
        aRows = 0;
        xtrader_reset (MHFile::CountRows(aFileName()));
        MHCSVFile file (aFileName(), '\t', 1000000);

        while (file.ReadRow()) {
            aRows++;
            try {
                if (file.Cols() < 2) throw ("");
                if (file.GetChar(0) == '#') throw ("");

                switch (*(file.GetString(0) + 1)) {
                    case 'a':
                        importAccount (*file.GetString(0), &file);
                        break;

                    case 'e':
                        importSet (*file.GetString(0), &file);
                        break;

                    case 'g':
                        importGuard (*file.GetString(0), &file);
                        break;

                    case 'i':
                        importPref (*file.GetString(0), &file);
                        break;

                    case 'p':
                        importPrice (*file.GetString(0), &file);
                        break;

                    case 'o':
                        importPortfolio (*file.GetString(0), &file);
                        break;

                    case 's':
                        importSecurity (*file.GetString(0), &file);
                        break;

                    case 't':
                        importSetItem (*file.GetString(0), &file);
                        break;

                    default:
                        critical++;
                        aErrors++;
                        break;
                }
            }
            catch (const char* message) {
                if (message && strlen(message) > 0)
                    DBFactory::WriteLog ("%s: row=%d (%s)",  STRING(ERROR_), aRows, message);
                aErrors++;
            }
            xtrader_add(1);
            if (xtrader_abort())
                throw (STRING(ERROR_CANCEL_IMPORT));
            if (critical > 20)
                throw (STRING(ERROR_IMPORT));
        }
    }
    catch (const char* message) {
        DBFactory::WriteLog ("%s", message);
        cleanup ();
        throw (message);
    }
    cleanup ();
}



/**
*  Import Account data
*  @param char      - '+' for importing, '-' for erasing
*  @param MHVector* - Vector with row data
*/
void Import::importAccount (char action, MHCSVFile* v) {
    if (v->Cols() != 4) throw ("Wrong number of columns for account row");
    aAccounts = DBFactory::LoadAccount();

    if (action == '+') {
        Account* a1 = new Account (0, CHAR(1), DOUBLE(2), CHAR(3));
        Account* a2 = (Account*) aAccounts->Find (a1, MHVector::FIND_EXACT, Account::SORT_NAME, 0);

        a1->IsChanged (true);
        if (a2) {
               a1->SetID (a2->GetID());
            *a2 = *a1;
            delete a1;
        }
        else {
            a1->SetID (DBFactory::GetUniqueID());
            aAccounts->InsertSorted (a1);
        }
        Account::Save (aAccounts);
    }
}



/**
*  Import Guard data
*  @param char      - '+' for importing, '-' for erasing
*  @param MHVector* - Vector with row data
*/
void Import::importGuard (char action, MHCSVFile* v) {
    Security* security;
    aSecurities = DBFactory::LoadSecurity();

    if (v->Cols() != 4) throw ("Wrong number of columns for price watch row");

    Security s (0, CHAR(1));
    security = (Security*) aSecurities->Find (&s, MHVector::FIND_EXACT, Security::SORT_CODE, 0);
    aGuards  = DBFactory::LoadGuard();

    if (!security) throw ("Security can't be found for price guard row");

    if (action == '+') {
        Guard* g = new Guard (security->GetID(), (Guard::TYPE)INT(2), DOUBLE(3));
        g->IsChanged (true);
        aGuards->InsertSorted (g);
    }
}



/**
*  Import Portfolio data
*  @param char      - '+' for importing, '-' for erasing
*  @param MHVector* - Vector with row data
*/
void Import::importPortfolio (char action, MHCSVFile* v) {
    Account*  account;
    Security* security;

    if (v->Cols() != 9) throw ("Wrong number of columns for portfolio row");
    aAccounts   = DBFactory::LoadAccount();
    Account a (CHAR(1));
    account     = (Account*) aAccounts->Find (&a, MHVector::FIND_EXACT, Account::SORT_NAME, 0);
    aSecurities = DBFactory::LoadSecurity();

    if (account) {
        Security s (0, CHAR(4));
        security = (Security*) aSecurities->Find (&s, MHVector::FIND_EXACT, Security::SORT_CODE, 0);
        if (aAccount == 0 || *account != *aAccount) {
            //  If new account found save old portfolio list
            if (aAccount != 0) Portfolio::Save (aAccount, &aPortfolios);
            aPortfolios.Erase ();
            aPortfolios.Resize (500);
            Portfolio::Load (account, &aPortfolios);
            aAccount = account;
        }

        if (action == '+') {
            Portfolio* p = new Portfolio (CHAR(2), (Portfolio::EVENT)INT(3), (security) ? security->GetID() : 0, DOUBLE(5), DOUBLE(6), DOUBLE(7), 0, 0, CHAR(8));
            p->IsChanged (true);
            aPortfolios.InsertSorted (p);
        }
        Portfolio::Save (aAccount, &aPortfolios);
    }
    else
        throw ("Can't find account for this portfolio transaction");
}



/**
*  Import Pref data
*  @param char      - '+' for importing, '-' for erasing
*  @param MHVector* - Vector with row data
*/
void Import::importPref (char action, MHCSVFile* v) {
    if (v->Cols() != 3) throw ("Wrong number of columns for ini row");
    aIni = DBFactory::LoadPref();

    if (action == '+') {
        Pref* a1 = new Pref (CHAR(1), CHAR(2));
        Pref* a2 = (Pref*)aIni->Find (a1, 0, 0, 0);

        a1->IsChanged (true);
        if (a2) {
            *a2 = *a1;
            delete a1;
        }
        else
            aIni->InsertSorted (a1);
    }
}



/**
*  Import Price data
*  @param char      - '+' for importing, '-' for erasing
*  @param MHVector* - Vector with row data
*/
void Import::importPrice (char action, MHCSVFile* v) {
    Security* security;

    if (v->Cols() != 9) throw ("Wrong number of columns for price row");
    aSecurities = DBFactory::LoadSecurity();
    Security s (0, CHAR(1));
    security    = (Security*) aSecurities->Find (&s, MHVector::FIND_EXACT, Security::SORT_CODE, 0);

    if (security) {
        if (aSecurity == 0 || *security != *aSecurity) {
            //  If new security found save old price list
            if (aSecurity != 0) Price::Save (aSecurity, &aPrices, false);
            aPrices.Erase ();
            aPrices.Resize (500);
            Price::LoadAll (security, &aPrices);
            aSecurity = security;
        }

        if (action == '+') {
            Price* p1 = new Price (CHAR(2), (Price::TYPE)INT(3), DOUBLE(4),DOUBLE(5), DOUBLE(6), DOUBLE(7), DOUBLE(8));
            Price* p2 = (Price*) aPrices.Find (p1, MHVector::FIND_EXACT, Price::SORT_DATE_TYPE, 0);

            p1->IsChanged (true);
            if (p2) {
                *p2 = *p1;
                delete p1;
            }
            else {
                // Only one date with type PRICE or CLOSED may exist at the same time
                // Try to delete the "opposite" type
                if (p1->aType == Price::PRICE) {
                    Price erase (p1->aDate, Price::CLOSED, 0.0);
                    p2 = (Price*) aPrices.Find (&erase, MHVector::FIND_EXACT, Price::SORT_DATE_TYPE, 0);
                }
                else if (p1->aType == Price::CLOSED) {
                    Price erase (p1->aDate, Price::PRICE, 0.0);
                    p2 = (Price*) aPrices.Find (&erase, MHVector::FIND_EXACT, Price::SORT_DATE_TYPE, 0);
                }
                if (p2)
                    aPrices.Erase(p2);
                aPrices.InsertSorted (p1, Price::SORT_DATE_TYPE);
            }
        }
    }
    else
        throw ("Can't find security for this price");
}



/**
*  Import Security data
*  @param char      - '+' for importing, '-' for erasing
*  @param MHVector* - Vector with row data
*/
void Import::importSecurity (char action, MHCSVFile* v) {
    if (v->Cols() < 3) throw ("Wrong number of columns for security row");
    aSecurities = DBFactory::LoadSecurity();

    if (action == '+') {
        Security* s1 = new Security (0, CHAR(1), CHAR(2), CHAR(3), CHAR(4), DOUBLE(5));
        Security* s2 = (Security*)aSecurities->Find (s1, MHVector::FIND_EXACT, Security::SORT_CODE, 0);

        s1->IsChanged (true);
        if (s2) {
            s1->SetID (s2->GetID());
            *s2 = *s1;
            delete s1;
        }
        else {
            s1->SetID (DBFactory::GetUniqueID());
            //if (aSecurities->Search (s1, MHVector::FIND_EXACT, Security::SORT_NAME, 0)) {
            //    delete s1;
            //    throw "Security name exist already";
            //}
            aSecurities->InsertSorted (s1, Security::SORT_CODE);
        }
        //Security::Save (aSecurities);
    }
}



/**
*  Import Set data
*  @param char      - '+' for importing, '-' for erasing
*  @param MHVector* - Vector with row data
*/
void Import::importSet (char action, MHCSVFile* v) {
    if (v->Cols() != 2) throw ("Wrong number of columns for set row");
    aSets = DBFactory::LoadSet();

    if (action == '+') {
        Set* s1 = new Set (0, CHAR(1));
        Set* s2 = (Set*)aSets->Find (s1, 0, 0, 0);

        s1->IsChanged (true);
        if (s2) {
            s1->SetID (s2->GetID());
            *s2 = *s1;
            delete s1;
            SetItem::Remove (s2);
        }
        else {
            s1->SetID (DBFactory::GetUniqueID());
            aSets->InsertSorted (s1);
        }
    }
}



/**
*  Import Portfolio data
*  @param char      - '+' for importing, '-' for erasing
*  @param MHVector* - Vector with row data
*/
void Import::importSetItem (char action, MHCSVFile* v) {
    Set*      set;
    Security* security;

    if (v->Cols() != 3) throw ("Wrong number of columns for set item row");
    aSets       = DBFactory::LoadSet();
    Set s (0, CHAR(1));
    set         = (Set*) aSets->Find (&s, 0, 0, 0);
    aSecurities = DBFactory::LoadSecurity();

    if (set) {
        Security s (0, CHAR(2));
        security = (Security*) aSecurities->Find (&s, MHVector::FIND_EXACT, Security::SORT_CODE, 0);
        if (security) {
            if (aSet == 0 || *set != *aSet) {
                //  If new set found save old set item list
                if (aSet != 0) SetItem::Save (aSet, &aSetItems);
                aSetItems.Erase();
                DBFactory::LoadSetItem (set, &aSetItems);
                aSet = set;
            }

            if (action == '+') {
                SetItem* i1 = new SetItem (security->GetID());
                SetItem* i2 = (SetItem*)aSetItems.Find (i1, MHVector::FIND_EXACT, SetItem::SORT_ID, 0);

                i1->IsChanged (true);
                if (i2) {
                    *i2 = *i1;
                    delete i1;
                }
                else
                    aSetItems.InsertSorted (i1, SetItem::SORT_ID);
            }
        }
    }
    else
        throw ("Can't find security list for this set item");
}
