/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/Portfolio.h>
#include <db/Account.h>
#include <db/DBFactory.h>
#include <db/Security.h>
#include <Resource.h>
#include <MHCSVFile.h>
#include <MHUtil.h>



/**
*  buy/sell constructor
*  @param const Portfolio& - Object to copy
*/
Portfolio::Portfolio (const char* date, EVENT event, int sid, double volume, double price, double courtage, int aid, int tid, const char* memo) :
MH() {
    aDate          = date;
    aEvent         = event;
    aSecurityID    = sid;
    aVolume        = volume;
    aPrice         = price;
    aCourtage      = courtage;
    aAccountID     = aid;
    aTransferID    = tid;
    aOldAccountID  = 0;
    aOldTransferID = 0;
    aMemo          = memo;
}



/**
*  copy constructor
*  @param const Portfolio& - Object to copy
*/
Portfolio::Portfolio (const Portfolio& x) :
MH() {
    IsChanged (((Portfolio&)x).IsChanged());
    aDate          = x.aDate;
    aEvent         = x.aEvent;
    aSecurityID    = x.aSecurityID;
    aVolume        = x.aVolume;
    aPrice         = x.aPrice;
    aCourtage      = x.aCourtage;
    aAccountID     = x.aAccountID;
    aTransferID    = x.aTransferID;
    aOldAccountID  = x.aOldAccountID;
    aOldTransferID = x.aOldTransferID;
    aMemo          = x.aMemo;
}



/**
*  Operator =
*  @param const Portfolio& - Object to copy
*/
Portfolio& Portfolio::operator=(const Portfolio& x) {
    if (this == &x) return *this;

    IsChanged (((Portfolio&)x).IsChanged());
    aDate       = x.aDate;
    aEvent      = x.aEvent;
    aSecurityID = x.aSecurityID;
    aVolume     = x.aVolume;
    aPrice      = x.aPrice;
    aCourtage   = x.aCourtage;
    aAccountID  = x.aAccountID;
    aTransferID = x.aTransferID;
    aMemo       = x.aMemo;

    return *this;
}



/**
*  Export data to file
*/
void Portfolio::Export (Account* account, MHVector* ve, FILE* file) {
    Portfolio*  i;
    Security*   s;
    char        buffer[200];

    for (int f = 0; f < ve->Size(); f++) {
        i = (Portfolio*) (*ve)[f];
        s = i->LoadSecurity();

        snprintf (buffer, 199, "+o\t%s\t%s\t%d\t%s\t%f\t%f\t%f\t%s\n", account->GetAccount()->Get(), i->GetDate()->Get(), i->GetEvent(), ((s != 0) ? s->GetCode()->Get() : ""), i->GetVolume(), i->GetPrice(), i->GetCourtage(), i->GetMemo()->Get());
        fwrite (buffer, sizeof(char), strlen (buffer), file);
    }
}



/**
*
*/
const char* Portfolio::GetEventName(int event) {
    switch (event) {
        case EV_BUY:
            return STRING(BUY);
        case EV_SELL:
            return STRING(SELL);
        case EV_TAX:
            return STRING(TAX);
        case EV_DIVIDEND:
            return STRING(DIVIDEND);
        case EV_LOAN:
            return STRING(LOAN);
        case EV_IN:
            return STRING(TRANSFER_IN);
        case EV_OUT:
            return STRING(TRANSFER_OUT);
        case EV_FEE:
            return STRING(FEE);
        case EV_TRANSFER:
            return STRING(TRANSFER);
        case EV_INTEREST:
            return STRING(INTEREST);
        default:
            return "";
    }
}



/**
*  Load a vector of Portfolio objects.
*  @param Account* - Account owner
*  @param MHVector*  - Vector
*/
void Portfolio::Load (Account* account, MHVector* ve) {
    MHString        name = DBFactory::Database();

    name += account->GetID();
    name += ".o";
    ve->Erase ();

    try {
        MHCSVFile file (name(), '\t');

        while (file.ReadRow()) {
            if (file.Cols() != 9)
                throw "Wrong number of columns in the file";
            ve->Push (new Portfolio (file.GetString(0), (EVENT)file.GetInt(1), file.GetInt(2), file.GetDouble(3), file.GetDouble(4), file.GetDouble(5), file.GetInt(6), file.GetInt(7), file.GetString(8)));
        }
    }
    catch (const char* x) {
        ve->Erase ();
        if (MHFile::Exist (name()) == true)
            DBFactory::WriteLog ("Loading portfolio failed file=%s, error=%s", name(), x);
    }
}



/**
*
*/
Security*  Portfolio::LoadSecurity() {
    aSecurity.SetID(0);
    aSecurity.SetName("");

    MHVector*  v = DBFactory::LoadSecurity();
    Security   s1 (aSecurityID);
    Security*  s2 = (Security*) v->Find (&s1, MHVector::FIND_EXACT, Security::SORT_ID, 0);
    if (s2)
        aSecurity = *s2;
    return &aSecurity;
}



/**
*
*/
void  Portfolio::LoadSecurities(MHVector* v) {
    for (int f = 0; f < v->Size(); f++)
        ((Portfolio*)(*v)[f])->LoadSecurity();
}


/**
*  Save a vector of Portfolio objects.
*  Throws Exception at failure
*  @param Account* - Account owner
*  @param MHVector*  - Vector
*/
void Portfolio::Save (Account* account, MHVector* ve) {
    Portfolio*      i;
    MHString        name = DBFactory::Database();

    ve->Sort();
    name += account->GetID();
    name += ".o";

    try {
        MHCSVFile file;
        char      buff[400];

        for (int f = 0; f < ve->Size(); f++) {
            i = (Portfolio*) (*ve)[f];
            sprintf (buff, "%s\t%d\t%d\t%.2f\t%.2f\t%.2f\t%d\t%d\t%s",
                i->GetDate()->Get(),
                i->GetEvent(),
                i->GetSecurityID(),
                i->GetVolume(),
                i->GetPrice(),
                i->GetCourtage(),
                i->GetAccountID(),
                i->GetTransferID(),
                i->GetMemo()->Get());
            file.WriteString (buff);
        }
        file.WriteFile (name());
        MH::ClearChanged (ve);
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving portfolio failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }
}
