/**
*  Copyright Mikael H?gdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "db/Account.h"
#include "db/DBFactory.h"
#include "db/Price.h"
#include "db/Portfolio2.h"
#include "db/Security.h"
#include "report/PortfolioValue.h"
#include "Progress.h"
#include "Resource.h"
#include <MHDate.h>
#include <MHDebug.h>



/**
*  Copy constructor.
*  @param const Portfolio2& - Object to copy
*/
Portfolio2::Portfolio2 (const Portfolio2& x) : Portfolio(x) {
    aCurrPrice     = x.aCurrPrice;
    aCurrPriceDate = x.aCurrPriceDate;
    aDivPrice      = x.aDivPrice;
    aDivVolume     = x.aDivVolume;
    aName          = x.aName;
}



/**
*  Operator =
*  @param const Portfolio& - Object to copy
*/
Portfolio2& Portfolio2::operator=(const Portfolio2& x) {
    if (this == &x) return *this;

    ((Portfolio&)*this) = x;
    aCurrPrice     = x.aCurrPrice;
    aCurrPriceDate = x.aCurrPriceDate;
    aDivPrice      = x.aDivPrice;
    aDivVolume     = x.aDivVolume;
    aName          = x.aName;

    return *this;
}



/**
*  Set new average price for an portfolio transaction
*  @param Portfolio* - New trade to add
*/
void Portfolio2::AddNewPrice (Portfolio2* r2) {
    switch (r2->aEvent) {
        case EV_DIVIDEND:
        if (r2->aDivVolume > 0.0000001) {
                aDivPrice   = ((r2->aDivPrice * r2->aDivVolume) + (aDivPrice * aDivVolume)) / (r2->aDivVolume + aDivVolume);
                aDivVolume += r2->aDivVolume;
        }
            break;

        case EV_BUY:
        if (r2->aVolume > 0.0000001) {
                aPrice      = ((r2->aPrice * r2->aVolume) + (aPrice * aVolume)) / (r2->aVolume + aVolume);
                aVolume    += r2->aVolume;
                aCurrPrice  = r2->aPrice;
                aCourtage  += r2->aCourtage;
                aDate       = r2->aDate;
        }
            break;

        case EV_SELL:
        if (r2->aVolume > 0.0000001) {
                aVolume    -= r2->aVolume;
                aCurrPrice  = r2->aPrice;
                aCourtage  += r2->aCourtage;
                aDate       = r2->aDate;
            }
        break;

        case EV_TAX:
        if (aDivVolume > 0.0000001)
                aDivPrice = ((aDivPrice * aDivVolume - r2->aPrice) / aDivVolume);
            break;

        default:
            break;
    }

    //  Reset price and dividend if buy and sell is even
    if (aVolume < 0.000001) {
        aVolume = 0;
        aPrice  = 0;
        aDivPrice  = 0;
        aDivVolume = 0;
    }
}



/**
*  Add new transaction to portfolio calculations.
*  Set new buy price if current date is greater than buy date.
*  @param const char*    - Current date
*  @param Portfolio2*    - New transaction
*  @param PortfolioValue - Portfolio value struct
*  @param MHVector*      - List with current buys, add new trade to list
*/
void Portfolio2::AddNewTrade (const char* startdate, Portfolio2* r2, PortfolioValue* value, MHVector* in) {
    DBG ("Portfolio2::AddNewTrade")
    DBG_LOG4 ("%s, %s, %d, %d", startdate, r2->GetDate()->Get(), r2->GetEvent(), r2->GetSecurityID())

    value->aCurrCash += r2->CashValue();

    switch (r2->GetEvent()) {
        case Portfolio::EV_BUY:
        case Portfolio::EV_SELL:
            if (r2->GetSecurityID() > 0) {
                if (*(r2->GetDate()) < startdate) {
                    r2->LoadBuyPrice (startdate);
                    r2->SetCourtage (0);
                }
                in->Push (new Portfolio2 (*r2));
            }
            break;

        case Portfolio::EV_DIVIDEND: {
            if (r2->GetSecurityID() != 0) {
                Portfolio2* p2 = new Portfolio2 (*r2);
                p2->SetDivPrice (r2->GetPrice());
                p2->SetDivVolume (r2->GetVolume());
                p2->SetPrice (0);
                p2->SetVolume (0);

                if (*(r2->GetDate()) < startdate) {
                    p2->SetDivVolume (0.0);
                    p2->SetDivPrice (0.0);
                }
                in->Push (p2);
            }
            break;
        }

        case Portfolio::EV_TAX:
            if (r2->GetSecurityID() != 0)
                in->Push (new Portfolio2 (*r2));
            break;

        case Portfolio::EV_IN:
            value->AddCash (r2, in);
            break;

        case Portfolio::EV_OUT:
            value->RemoveCash (r2, in);
            break;

        default:
            break;
    }

    if (*r2->GetDate() < startdate)
        value->Reset (startdate, in);
    else
        value->Log ();
}



/**
*  Return current cash value of trade
*  @return double - Value
*/
double Portfolio2::CashValue () {
    switch (GetEvent()) {
        case EV_IN:
        case EV_INTEREST:
            return aPrice;

        case EV_SELL:
        case EV_DIVIDEND:
            return value (SELL_VALUE);

        case EV_BUY:
            return -(value (BUY_VALUE));

        case EV_TAX:
        case EV_OUT:
        case EV_LOAN:
        case EV_FEE:
            return -(aPrice);

        default:
            return 0;
    }
}






/**
*  Filter trades from portfolio account. Count current cash left in portfolio also.
*  @param MHVector* - The portfolio vector
*  @param MHVector* - The result vector, contains only BUY, SELL, DIVIDEND and TAX transactions
*/
void Portfolio2::GetBuySell (MHVector* in, MHString* stopDate, MHVector* out) {
    for (int f = 0; f < in->Size(); f++) {
        Portfolio2* p2 = (Portfolio2*) in->Get (f);

        switch (p2->GetEvent()) {
            case Portfolio::EV_BUY:
            case Portfolio::EV_SELL:
            case Portfolio::EV_DIVIDEND:
            case Portfolio::EV_TAX:
                if (p2->GetSecurityID() > 0 && *(p2->GetDate()) <= *stopDate)
                    out->Push (new Portfolio2 (*p2));
                break;

            default:
                break;
        }
    }
}



/**
*  Group trades by code. All "inactive" trades are removed.
*  Inactive trades are those with total volume=0.
*  @param MHVector* - Vector with trades
*  @param MHVector* - Vector list with grouped trades
*/
void Portfolio2::GroupTrades (MHVector* in, MHVector* out) {
    DBG ("Portfolio2::GroupTrades")

    int             f, e;
    Portfolio2*     pin;
    Portfolio2*     pout;
    MHVector        tmp;

    //  Start to group trades
    for (f = 0; f < in->Size(); f++) {
        pin  = (Portfolio2*) (*in)[f];
        pout = 0;

        //  Check if trade with same object exist, if so use that trade, otherwise add new trade
        for (e = 0; e < tmp.Size(); e++) {
            pout = (Portfolio2*) tmp[e];
            if (pin->GetSecurityID() == pout->GetSecurityID()) break;
            pout = 0;
        }

        if (pout)
            pout->AddNewPrice (pin);
        else
            tmp.Push (new Portfolio2 (*pin));
        DBG_LOG7 ("1:%s, %03d, %d, %f, %f, %f, %f", pin->GetDate()->Get(), pin->GetSecurityID(), pin->GetEvent(), pin->GetPrice(), pin->GetVolume(), pin->GetDivPrice(), pin->GetDivVolume());
    }

    //  Erase all transactions with volume == 0 which are in outlist from inlist, performance thing, dont fetch price for these
    for (f = 0; f < tmp.Size(); f++) {
        pout = (Portfolio2*) tmp[f];
        if (pout->GetVolume() > 0.000001)
            out->Push (new Portfolio2 (*pout));
    }
    tmp.Erase ();
    #ifdef BUGG
        for (f = 0; f < out->Size(); f++) {
            pin = (Portfolio2*) out->Get(f);
            DBG_LOG7 ("2:%s, %03d, %d, %f, %f, %f, %f", pin->GetDate()->Get(), pin->GetSecurityID(), pin->GetEvent(), pin->GetPrice(), pin->GetVolume(), pin->GetDivPrice(), pin->GetDivVolume());
        }
    #endif
    DBG_LOG2 ("inlist=%d  outlist=%d", in->Size(), out->Size())
}



/**
*  Get data for portfolio from database.
*  @param Account*    Account to load transactions for
*  @param MHVector*   The result vector
*/
void Portfolio2::LoadPortfolio (Account* account, MHVector* out) {
    DBG ("Portfolio2::LoadPortfolio")
    MHVector ve;

    Portfolio::Load (account, &ve);
    DBG_LOG1 ("Portfolio size=%d", ve.Size())

    //  Add an dummy transaction if the account has an start balance
    if (account->GetBalance() > 0.00001) {
        Portfolio2* p2 = new Portfolio2();
        p2->SetDate ("19700101");
        p2->SetPrice (account->GetBalance ());
        p2->SetEvent (Portfolio::EV_IN);
        out->Push (p2);
    }

    for (int f = 0; f < ve.Size(); f++) {
        Portfolio* p = (Portfolio*) ve[f];
        out->Push (new Portfolio2(*p));
    }
    ve.Erase ();
}



/**
*  Get prices for a list of objects, but only for objects with BUY/SELL events.
*  Change both buy (if buy date is less than start date) and current prices.
*  @param const char* - Date
*  @param MHVector*   - The vector with Portfolio2 objects
*  @param bool        - True to set buy price otherwise current price is set
*/
void Portfolio2::loadPrices (const char* date, MHVector* in, bool current) {
    for (int f = 0; f < in->Size(); f++) {
        Portfolio2* r2    = (Portfolio2*) (*in)[f];
        MHString*   date2 = r2->GetDate();

        if (current) {
            if ((r2->GetEvent() == EV_BUY || r2->GetEvent() == EV_SELL) && r2->aVolume > 0.000001 && *date2 <= date) {
                r2->LoadCurrPrice (date);
            }
        }
        else {
            if ((r2->GetEvent() == EV_BUY || r2->GetEvent() == EV_SELL) && r2->aVolume > 0.000001 && *date2 < date) {
                r2->LoadBuyPrice (date);
            }
        }
    }
}



/**
*  Find and update price
*  @param const char* - Date to search for
*  @param bool        - true to set current price, false to set buy price
*  @return bool       - true if found
*/
bool Portfolio2::loadPrice (const char* date, bool current) {
    DBG ("Portfolio2::loadPrice")
    DBG_LOG3 ("portfoliodate=%s, date=%s, id=%d", aDate(), date, aSecurityID)

    MHVector*   vs  = DBFactory::LoadSecurity();
    Security    s (aSecurityID);
    Security*   sec = (Security*) vs->Find (&s, MHVector::FIND_EXACT, Security::SORT_ID, 0);
    MHVector*   vp  = DBFactory::LoadPrice (sec);

    if (vp && vp->Size() > 0) {
        Price     p2 (date, 0);
        Price*    p = (Price*) vp->Find (&p2, MHVector::FIND_BEFORE, Price::SORT_DATE, 0);

        #ifdef BUGG
            if (p) DBG_LOG2 ("found %s - %f", p->aDate, p->aClose)
        #endif

        if (p && *p >= aDate()) {
            if (current == true) {
                aCurrPrice     = p->aClose;
                aCurrPriceDate = p->aDate;
                DBG_LOG2 ("currPrice %s - %f", p->aDate, p->aClose)
                return true;
            }
            else {
                aPrice = p->aClose;
                aDate  = p->aDate;
                DBG_LOG2 ("Price %s - %f", p->aDate, p->aClose)
                return true;
            }
        }
    }
    DBG_LOG ("FAILED")
    return false;
}



/**
*  Calculate portfolio performance.
*  For every week calc. value of portfolio as a share. Start at 100.
*  @param MHVector*        - Trades
*  @param MHVector*        - Store vector for buy/sell transactions
*  @param MHVector*        - Vector with MHPrice objects, set to 0 to omit
*  @param MHVector*        - Vector with MHPrice objects, set to 0 to omit
*  @param MHString*        - Start date
*  @param MHString*        - Stop date
*  @param PortfolioValue*  - Performance struct
*/
bool Portfolio2::Performance (MHVector* trades, MHVector* buyList, MHVector* chartp, MHVector* chartv, MHString* start, MHString* stop, PortfolioValue* value) {
    DBG ("Portfolio2::Performance")

    int         nIndex = 0;
    int         nDays;
    MHDate      currDate (start->Get());
    MHDate      stopDate (stop->Get());
    MHDate      tmpDate;
    Portfolio2* r2;

    if (trades->Size() == 0 || *start >= *stop) return false;

    r2 = (Portfolio2*) (trades->Get(nIndex));
    tmpDate.Set (r2->GetDate()->Get());
    nDays = (stopDate.GetSecs() - tmpDate.GetSecs()) / (3600 * 24);

    xtrader_reset (nDays);

    //  Goto prev. friday so first performance number are done at the first friday same week as startdate
    currDate.AddDays (-7);
    currDate.GotoNextWeekday (MHDate::FRIDAY);
    if (stopDate.GetWeekday() < MHDate::FRIDAY)
        stopDate.GotoNextWeekday (MHDate::FRIDAY);

    DBG_LOG1 ("start   =%s", start->Get());
    DBG_LOG1 ("stop    =%s", stop->Get());
    DBG_LOG1 ("tmpDate =%s", tmpDate.Get (MHDate::LONG_DATE));
    DBG_LOG1 ("nDays   =%d", nDays);

    while (currDate <= stopDate) {
        DBG_LOG (" ");
        DBG_LOG1 ("%s", currDate.Get(MHDate::LONG_DATE));

        //  If no chart serie then skip all iterations for every week and jump to last date
        if (!chartp && !chartv) currDate = stopDate;

        //  Add trades until next friday (current stopdate)
        while (r2 && *(r2->GetDate()) <= currDate.Get(MHDate::LONG_DATE)) {
            Portfolio2::AddNewTrade (start->Get(), r2, value, buyList);
            r2 = (Portfolio2*) trades->Get (++nIndex);
        }

        value->Recalc (currDate.Get(MHDate::LONG_DATE), buyList);
        value->NewSharePrice ();
        value->Log ();

        //  Add chart points
        if (chartp) chartp->Push (new MHPrice (currDate.Get (MHDate::LONG_DATE), value->aShareVolume > 0.0000001 ? value->aSharePrice : 100.0));
        if (chartv) chartv->Push (new MHPrice (currDate.Get (MHDate::LONG_DATE), value->aCurrValue + value->aCurrCash));

        currDate.AddDays (7);
        xtrader_add(7);
        if (xtrader_abort()) return false;

        #ifdef BUGG
            for (int f = 0; f < buyList->Size(); f++) {
                Portfolio2* p2 = (Portfolio2*) buyList->Get(f);
                DBG_LOG7 ("buylist %s, %03d, %d, %f, %f, %f, %f", p2->GetDate()->Get(), p2->GetSecurityID(), p2->GetEvent(), p2->GetPrice(), p2->GetVolume(), p2->GetDivPrice(), p2->GetDivVolume());
            }
        #endif
    }
    return true;
}


/**
*  Return value for security.
*  @param  int    - VALUE, BUY_VALUE, SELL_VALUE
*  @return double - Value
*/
double Portfolio2::value (int type) {
    double val = 0;

    switch (type) {
        case VALUE:
            if (aVolume < 0.0000001)
                val = aPrice;
            else {
                if (aCurrPriceDate == "")
                    val = fabs(aPrice * aVolume);
                else
                    val = fabs(aCurrPrice * aVolume);
            }
            break;

        case BUY_VALUE:
            val = fabs(aPrice * aVolume) + aCourtage;
            break;

        case BUY_VALUE2:
            val = fabs(aPrice * aVolume);
            break;

        case SELL_VALUE:
            val = fabs(aPrice * aVolume) - aCourtage;
            break;

        case DIV_VALUE:
            val = aDivPrice * aDivVolume;
            break;
    }
    return val < 0.00001 ? 0.0 : val;
}
