/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/Price.h>
#include <db/DBFactory.h>
#include <Resource.h>
#include <MHCSVFile.h>



/**
*  Operator =
*  @param const Price& - Object to copy
*/
Price& Price::operator=(const Price& x) {
    if (this == &x) return *this;
    MHPrice::operator=(x);
    IsChanged (((Price&)x).IsChanged());
    aType = x.aType;
    return *this;
}



/**
*  Compare price date
*  @return int - 1,0,-1
*/
int Price::Compare (const MH* o, int type) {
    if (type == SORT_DATE)
        return MHPrice::Compare(o, SORT_DATE);
    else {
        char b[20];
        char c[20];
        sprintf (b, "%s%c", aDate, aType);
        sprintf (c, "%s%c", ((Price*)o)->aDate, ((Price*)o)->aType);
        return strcmp(b,c);
    }
}



/**
*  Export data to file
*/
void Price::Export (Security* sec, MHVector* ve, FILE* file) {
    Price*  i;
    char    buffer[200];

    for (int f = 0; f < ve->Size(); f++) {
        i = (Price*) (*ve)[f];
        snprintf (buffer, 199, "+p\t%s\t%s\t%d\t%f\t%f\t%f\t%f\t%f\n", sec->GetCode()->Get(), i->aDate, (int)i->aType, i->aHigh, i->aLow, i->aClose, i->aOpen, i->aVol);
        fwrite (buffer, sizeof(char), strlen (buffer), file);
    }
}



/**
*  Load a vector of Price objects.
*  @param bool        - true to load all data as it is, don't split and keep closed/split data
*  @param Security*   - Load price for this security object
*  @param MHVector*   - Destination vector for price data
*  @param MHVector*   - Destination vector for closed days, can be omitted
*/
void Price::load (bool raw, Security* sec, MHVector* ve, MHVector* ve2) {
    MHString    name = DBFactory::Database();
    MHVector    split;

    name += sec->GetID();
    name += ".p";
    ve->Erase ();
    ve->SetSortType (Price::SORT_DATE_TYPE);

    try {
        MHCSVFile file (name(), '\t');

        while (file.ReadRow()) {
            if (file.Cols() != 7)
                throw "Wrong number of columns in the file";
            if (raw == false) {
                if (file.GetInt(1) == PRICE)
                    ve->Push (new Price (file.GetString(0), PRICE, file.GetDouble(2), file.GetDouble(3), file.GetDouble(4), file.GetDouble(5), file.GetDouble(6)));
                 else if (file.GetInt(1) == SPLIT)
                    split.Push (new Price (file.GetString(0), SPLIT, file.GetDouble(2), file.GetDouble(3), file.GetDouble(4), file.GetDouble(5), file.GetDouble(6)));
                 else if (file.GetInt(1) == CLOSED && ve2)
                    ve2->Push (new Price (file.GetString(0), CLOSED, file.GetDouble(2), file.GetDouble(3), file.GetDouble(4), file.GetDouble(5), file.GetDouble(6)));
            }
            else
                ve->Push (new Price (file.GetString(0), (TYPE)file.GetInt(1), file.GetDouble(2), file.GetDouble(3), file.GetDouble(4), file.GetDouble(5), file.GetDouble(6)));
        }
    }
    catch (const char* x) {
        ve->Erase ();
        if (MHFile::Exist (name()) == true)
            DBFactory::WriteLog ("Loading price failed file=%s, error=%s", name(), x);
    }

    if (raw) return;

    //  Adjust prices for split
    int       curr  = split.Size();
    double    fact  = 1.0;
    double    fact2 = 1.0;
    Price*    p;
    Price*    s;

    s = (curr > 0) ? (Price*)split[--curr] : 0;

    for (int f = ve->Size() - 1; f >= 0 && curr >= 0; f--) {
        p = (Price*) (*ve)[f];

        //  Check if current split is inside date range
        while (s != 0 && *s >= *p) {
            fact  *= s->aHigh;
            fact2 *= s->aLow;
            s = (curr > 0) ? (Price*)split[--curr] : 0;
        }
        p->Adjust (fact, fact2);
    }
    split.Erase ();
    ve->SetSortType (Price::SORT_DATE);
    if (ve2)
        ve2->SetSortType (Price::SORT_DATE);
}



/**
*  Save a vector of Price objects.
*  Throws Exception at failure
*  @param MHVector& - Vector
*/
void Price::Save (Security* sec, MHVector* ve, bool sort) {
    Price*     i;
    MHString   name = DBFactory::Database();

    name += sec->GetID();
    name += ".p";
    if (sort) ve->Sort(SORT_DATE_TYPE);

    try {
        MHCSVFile file;
        char      buff[200];

        for (int f = 0; f < ve->Size(); f++) {
            i = (Price*) (*ve)[f];
            sprintf (buff, "%s\t%d\t%.2f\t%.2f\t%.2f\t%.2f\t%.0f", i->aDate, i->aType, i->aHigh, i->aLow, i->aClose, i->aOpen, i->aVol);
            file.WriteString (buff);
        }
        file.WriteFile (name());
        MH::ClearChanged (ve);
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving price failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }
}
