/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "gui/BaseView.h"
#include "gui/XTrader.h"
#include "db/Account.h"
#include "db/Security.h"
#include "db/Set.h"
#include "db/SetItem.h"
#include "Enums.h"
#include "Resource.h"
#include <MHDebug.h>
#include <MHMessage.h>
#include <MHVector.h>
#include <MHUtil.h>
#include <FL/Fl_Box.h>
#include <FL/Fl_Button.h>
#include <FL/Fl_Hold_Browser.h>




/**
*  BaseView constructor
*/
BaseView::BaseView (const char* workSpace, int type, int x, int y, int w, int h, const char* label) :
Fl_Group (x, y, w, h, label) {
    aType           = type;
    aBrowser1       = 0;
    aBrowser2       = 0;
    aList1          = new MHVector();
    aList2          = new MHVector();
    aLayout         = new Fl_BorderLayout (x, y, w, h);
    aToolbar        = new Fl_VerticalLayout (0, 0, XTrader::FontSize() * 8 + 4, 0);
    aEditLayout     = new Fl_HorizontalSplitLayout (0, 0, w, 0);
    aBrowserLayout  = new Fl_VerticalSplitLayout (0, 0, (type == NO_BROWSER || type == GUARD) ? 0 : XTrader::FontSize() * 12, 0);

    aToolbar->space (2, 2);
    aEditLayout->space (5);
    aBrowserLayout->space (4);
    aLayout->space (10, 4, 0, 4);

    aEditLayout->addLeft (aBrowserLayout);
    aLayout->add (Fl_Layout::WEST, aToolbar);
    aLayout->add (Fl_Layout::CENTER, aEditLayout);
    add (aLayout);
    labelsize (XTrader::FontSize());
    user_data (this);
    STRNCPY (aWorkSpaceName, workSpace, 99);
    aMem1 = aMem2 = 0;
}



/**
*  BaseView destructor
*  Delete all list objects
*/
BaseView::~BaseView () {
    aList1->Erase ();
    aList2->Erase ();
    delete aList1;
    delete aList2;
}



/**
*  Create 0 - 2 browsers in a split widget
*/
void BaseView::createBrowser () {
    switch (aType) {
        case PORTFOLIO:
        case SECURITY:
        case SET:
            aBrowser1 = new Fl_Hold_Browser (0, 0, 0, h());
            aBrowserLayout->addTop (aBrowser1);
            break;

        case SET_SETITEM:
            aBrowser1 = new Fl_Hold_Browser (0, 0, 0, int(h() * 0.4));
            aBrowser2 = new Fl_Hold_Browser (0, 0, 0, int(h() * 0.6));
            aBrowserLayout->addTop (aBrowser1);
            aBrowserLayout->addBottom (aBrowser2);
            break;

        default:
            break;
    }

    if (aBrowser1) {
        aBrowser1->textsize (XTrader::FontSize());
        aBrowser1->callback ((Fl_Callback*)BaseView::cbBrowser1_);
        aBrowser1->when (FL_WHEN_CHANGED);
        aBrowser1->user_data (this);
    }

    if (aBrowser2) {
        aBrowser2->textsize (XTrader::FontSize());
        aBrowser2->callback ((Fl_Callback*)BaseView::cbBrowser2_);
        aBrowser2->when (FL_WHEN_CHANGED);
        aBrowser2->user_data (this);
    }
}



/**
*  Create toolbar with buttons
*  @params const char* - one or many labels, one for every button
*/
void BaseView::createToolbar (const char* str, ...) {
    va_list ap;

    va_start (ap, str);
    while (str) {
        Fl_Button* butt = new Fl_Button (0, 0, 0, 25, str);
        XTrader::FixButton (butt);
        butt->callback ((Fl_Callback*)BaseView::cbButton_);
        butt->user_data (this);
        aToolbar->add (butt);
        str = va_arg (ap, char*);
    }
    va_end (ap);
}



/**
*  Disable all buttons
*/
void BaseView::disableButtons () {
    for (int f = 0; f < aToolbar->children(); f++) {
        Fl_Button* b = (Fl_Button*) aToolbar->child (f);
        b->deactivate ();
    }
}



/**
*  Enable button
*/
void BaseView::enableButton (const char* label) {
    for (int f = 0; f < aToolbar->children(); f++) {
        Fl_Button* b = (Fl_Button*) aToolbar->child (f);

        if (strcmp (b->label(), label) == 0) {
            b->activate ();
            break;
        }
    }
}



/**
*  Return data so view can be recreated later
*/
const char* BaseView::GetWorkSpace () {
    STRNCPY (aWorkSpace, aWorkSpaceName, 99);

    switch (aType) {
        case NO_BROWSER:
            break;

        case PORTFOLIO:
            if (aMem1) {
                Account*a = (Account*) aMem1;

                snprintf (aWorkSpace, 99, "%s:%d:%d", aWorkSpaceName, a->GetID(), 0);
            }
            break;

        case SECURITY:
            if (aMem1) {
                Security* s = (Security*) aMem1;

                snprintf (aWorkSpace, 99, "%s:%d:%d", aWorkSpaceName, s->GetID(), 0);
            }
            break;

        case SET:
            if (aMem1) {
                Set* set = (Set*) aMem1;

                snprintf (aWorkSpace, 99, "%s:%d:%d", aWorkSpaceName, set->GetID(), 0);
            }
            break;

        case SET_SETITEM:
            if (aMem1 && aMem2) {
                Set*     set     = (Set*) aMem1;
                SetItem* setItem = (SetItem*) aMem2;

                snprintf (aWorkSpace, 99, "%s:%d:%d", aWorkSpaceName, set->GetID(), setItem->GetSecurityID());
            }
            break;

        case GUARD:
            break;
    }

    return aWorkSpace;
}



/**
*  Load data into browser widgets
*/
void BaseView::loadBrowserData1 () {
    int f;

    switch (aType) {
        case PORTFOLIO:
            aList1->Erase ();
            aBrowser1->clear ();
            Account::Load (aList1);
            aList1->Sort (Account::SORT_NAME);
            for (f = 0; f < aList1->Size(); f++) {
                Account* a = (Account*) (*aList1)[f];
                aBrowser1->add (a->GetAccount()->Get());
            }
            break;

        case SECURITY:
            aList1->Erase ();
            aBrowser1->clear ();
            Security::Load (aList1);
            aList1->Sort (Security::SORT_NAME);
            for (f = 0; f < aList1->Size(); f++) {
                Security* s = (Security*) (*aList1)[f];
                aBrowser1->add (s->GetName()->Get());
            }
            break;

        case SET:
        case SET_SETITEM:
            aList1->Erase ();
            aBrowser1->clear ();
            Set::Load (aList1);
            aList1->Sort ();
            for (f = 0; f < aList1->Size(); f++) {
                Set* s = (Set*) (*aList1)[f];
                aBrowser1->add (s->GetSet()->Get());
            }
            break;

        default:
            break;
    }
}



/**
*  Load data into browser widgets
*/
void BaseView::loadBrowserData2 () {
    int f;

    switch (aType) {
        case SET_SETITEM: {
            if (aBrowser1->value () == 0 || aBrowser1->value () > aList1->Size()) return;
            Set* set = (Set*) (*aList1)[aBrowser1->value() - 1];
            aList2->Erase ();
            aBrowser2->clear ();

            SetItem::Load (set, aList2);
            aList2->Sort (SetItem::SORT_SECURITY);
            SetItem::LoadSecurities (aList2);

            for (f = 0; f < aList2->Size(); f++) {
                SetItem* s = (SetItem*) (*aList2)[f];
                aBrowser2->add (s->GetSecurity()->GetName()->Get());
            }
            aMem1 = set;
            aMem2 = 0;
            break;
        }

        default:
            break;
    }
}



/**
*  Try to restore the workspace. Either when starting XTrader or to reload data.
*  @param const char* - Format is "classname:db_id:db_id"
*/
void BaseView::LoadWorkSpace (const char* ws) {
    MHString    s;
    MHVector    ve;
    int         id1 = 0;
    int         id2 = 0;

    s = ws;
    s.Split(':', &ve);

    if (ve.Size() == 1 && aType != GUARD) {
        ve.Erase ();
        return;
    }

    if (ve.Size() >= 2) {
        MHString* s2 = (MHString*) ve.Get(1);
        id1 = s2->ToInt();
    }

    if (ve.Size() == 3) {
        MHString* s2 = (MHString*) ve.Get(2);
        id2 = s2->ToInt();
    }

    ve.Erase ();

    switch (aType) {
        case NO_BROWSER:
            break;

        case PORTFOLIO:
            if (id1) {
                for (int f = 0; f < aList1->Size(); f++) {
                    Account* a = (Account*) (*aList1)[f];
                    if (a->GetID() == id1) {
                        aBrowser1->value (f + 1);
                        cbBrowser1 ();
                        break;
                    }
                }
            }
            break;

        case SECURITY:
            if (id1) {
                for (int f = 0; f < aList1->Size(); f++) {
                    Security* s = (Security*) (*aList1)[f];
                    if (s->GetID() == id1) {
                        aBrowser1->value (f + 1);
                        cbBrowser1 ();
                        break;
                    }
                }
            }
            break;

        case SET:
            if (id1) {
                for (int f = 0; f < aList1->Size(); f++) {
                    Set* s = (Set*) (*aList1)[f];
                    if (s->GetID() == id1) {
                        aBrowser1->value (f + 1);
                        cbBrowser1 ();
                        break;
                    }
                }
            }
            break;

        case SET_SETITEM:
            if (id1 && id2) {
                for (int f = 0; f < aList1->Size(); f++) {
                    Set* s = (Set*) (*aList1)[f];
                    if (s->GetID() == id1) {
                        aBrowser1->value (f + 1);
                        cbBrowser1 ();
                        break;
                    }
                }

                for (int g = 0; g < aList2->Size(); g++) {
                    SetItem* s = (SetItem*) (*aList2)[g];
                    if (s->GetSecurityID() == id2) {
                        aBrowser2->value (g + 1);
                        cbBrowser2 ();
                        break;
                    }
                }
            }
            break;

        case GUARD:
            cbBrowser1 ();
            break;
    }
}



/**
*  Message from some object, check out if for us
*/
void BaseView::Message (const MH* sender, const char* m1, const char* m2, const char* m3, void* m4) {
    if (DBFactory::GetPrefIntValue (STRING(PREF_AUTORELOAD), XTRADER_NO, XTRADER_YES, XTRADER_YES) == XTRADER_YES) {
        if (strcmp (m2, "changed") == 0) {
            const char* ws = GetWorkSpace ();

            loadBrowserData1 ();
            loadBrowserData2 ();
            LoadWorkSpace (ws);
        }
    }
}



/**
*  Resize edit view and all children
*/
void BaseView::resize (int x, int y, int w, int h) {
    DBG ("BaseView::resize")

    DBG_LOG1 ("%s", label())
    Fl_Group::resize (x, y, w, h);
    DBG_LOG1 ("%s", label())
    aLayout->resize (x, y, w, h);
    DBG_LOG4 ("BaseView         %4d, %4d, %4d, %4d", this->x(), this->y(), this->w(), this->h())
    DBG_LOG4 ("aLayout          %4d, %4d, %4d, %4d", aLayout->x(),  aLayout->y(),  aLayout->w(),  aLayout->h())
    DBG_LOG4 ("aEditLayout      %4d, %4d, %4d, %4d", aEditLayout->x(),    aEditLayout->y(),    aEditLayout->w(),    aEditLayout->h())
    DBG_LOG4 ("aBrowserLayout   %4d, %4d, %4d, %4d", aBrowserLayout->x(), aBrowserLayout->y(), aBrowserLayout->w(), aBrowserLayout->h())
}

