/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "gui/EditView.h"
#include "gui/XTrader.h"
#include "db/Account.h"
#include "db/DBFactory.h"
#include "db/Security.h"
#include "db/Set.h"
#include "db/SetItem.h"
#include "table/PortfolioData.h"
#include "table/PriceData.h"
#include "table/SetItemData.h"
#include "Enums.h"
#include "Resource.h"
#include <MHVector.h>
#include <FL/Fl.h>
#include <FL/Fl_Button.h>
#include <fl/Fl_Layout.h>
#include <FL/Fl_Hold_Browser.h>
#include <fl/Fl_Table.h>
#include <fl/Fl_Table_Data.h>



/**
*  BaseView constructor
*/
EditView::EditView (const char* workSpace, int type, Fl_Table_Data* data, int x, int y, int w, int h, const char* label) :
BaseView (workSpace, type, x, y, w, h, label) {
    aTable = new Fl_Table (XTrader::GetApp(), data, 0, 0, w, 0);
    aEditLayout->addRight (aTable);

    createToolbar (STRING(NEW), STRING(ERASE), STRING(SAVE), STRING(FIND_WORD), STRING(FIND_NEXT), STRING(CLOSE_), 0);
    createBrowser ();
    loadBrowserData1 ();
    loadBrowserData2 ();
    enableButtons ();
    aTable->event_handler (this);
    Fl::focus (aTable);
}



/**
*  Destroy view
*/
EditView::~EditView () {
}



/**
*  Check if view can be closed
*/
bool EditView::CanClose () {
    if (aTable->dirty()) {
        switch (fl_choice (STRING(ASK_CLOSE_DIRTY), STRING(CLOSE_), STRING(CLOSE_SAVE), STRING(CANCEL))) {
            case 0:
                return true;

            case 1:
                if (aTable->save() == false) {
                    fl_alert (STRING(ERROR_TABLE_SAVE));
                    return false;
                }
                return true;

            case 2:
                return false;
        }
    }
    return true;
}


/**
*  Callback for browser
*/
void EditView::cbBrowser1 () {
    aMem1 = aMem2 = 0;

    switch (aType) {
        case PORTFOLIO: {
            if (aBrowser1->value () == 0 || aBrowser1->value () > aList1->Size()) return;
            if (CanClose() == false) return;
            Account* a = (Account*) (*aList1)[aBrowser1->value() - 1];
            aTable->data (new PortfolioData (a));
            aMem1 = a;
            break;
        }

        case SET: {
            if (aBrowser1->value () == 0 || aBrowser1->value () > aList1->Size()) return;
            if (CanClose() == false) return;
            Set* set = (Set*) (*aList1)[aBrowser1->value() - 1];
            aTable->data (new SetItemData (set));
            aMem1 = set;
            break;
        }

        case SET_SETITEM: {
            if (CanClose() == false) return;
            loadBrowserData2 ();
            break;
        }
    }
    enableButtons ();
    Fl::focus (aTable);
}



/**
*  Callback for browser
*/
void EditView::cbBrowser2 () {
    aMem2 = 0;

    switch (aType) {
        case SET_SETITEM: {
            if (aBrowser2->value () == 0 || aBrowser2->value () > aList2->Size()) return;
            if (CanClose() == false) return;
            SetItem*  item  = (SetItem*) (*aList2)[aBrowser2->value() - 1];
            aTable->data (new PriceData (new Security (*item->GetSecurity())));
            aMem2 = item;
            break;
        }
    }
    enableButtons ();
    Fl::focus (aTable);
}



/**
*  Callback for buttons
*/
void EditView::cbButton (Fl_Button* button) {
    try {
        if (strcmp (button->label(), STRING(CLOSE_)) == 0)
            XTrader::Close (this);
        else if (strcmp (button->label(), STRING(ERASE)) == 0)
            aTable->erase ();
        else if (strcmp (button->label(), STRING(FIND_NEXT)) == 0)
            aTable->find (true);
        else if (strcmp (button->label(), STRING(FIND_WORD)) == 0)
            aTable->find (false);
        else if (strcmp (button->label(), STRING(NEW)) == 0)
            aTable->add ();
        else if (strcmp (button->label(), STRING(SAVE)) == 0)
            aTable->save();
    }
    catch (const char* x) {
        fl_alert (x);
    }

    enableButtons ();
    Fl::focus (aTable);
}



/**
*  Enable/disable buttons
*/
void EditView::enableButtons () {
    disableButtons ();

    Fl_Table_Data* data = aTable->data();
    if (data) {
        data->stripe(fl_gray_ramp(240 * (FL_NUM_GRAY - 1) / 255));
        enableButton (STRING(NEW));
        if (data->rows() > 0) {
            enableButton (STRING(ERASE));
            enableButton (STRING(FIND_WORD));
            enableButton (STRING(FIND_NEXT));
        }
        if (aTable->dirty()) enableButton (STRING(SAVE));
    }
    enableButton (STRING(CLOSE_));
    aToolbar->redraw ();
    Fl::focus (aTable);
}



/**
*
*/
int EditView::handle (int event) {
    switch (event) {
        case FL_HIDE:
            aTable->quit_edit_mode (false);
            return Fl_Group::handle (event);

        case FL_SHOW:
            return Fl_Group::handle (event);

        case FL_DATA_CHANGE:
        case FL_DATA_SAVE:
            enableButtons ();
            return 1;

        case FL_DATA_ERROR:
            XTrader::SetInfo (STRING(ERROR_DATA_ERROR));
            return 1;

        case FL_DATA_SAVE_ERROR:
            XTrader::SetInfo (STRING(ERROR_TABLE_SAVE));
            return 1;

        case FL_KEYDOWN:
            return aTable->handle (event);

        default:
            return Fl_Group::handle (event);
    }
}
