/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "gui/ReportView.h"
#include "gui/XTrader.h"
#include "db/Account.h"
#include "db/DBFactory.h"
#include "db/Security.h"
#include "db/Set.h"
#include "dlg/DateDlg.h"
#include "dlg/SelectDlg.h"
#include "report/BaseReport.h"
#include "report/PortfolioReport.h"
#include "report/PriceReport.h"
#include "table/ReportData.h"
#include "Enums.h"
#include "Resource.h"
#include <MHDate.h>
#include <MHVector.h>
#include <FL/Fl.h>
#include <FL/fl_ask.h>
#include <FL/fl_draw.h>
#include <FL/Fl_Button.h>
#include <FL/Fl_File_Chooser.h>
#include <fl/Fl_Layout.h>
#include <FL/Fl_Hold_Browser.h>
#include <fl/Fl_Table.h>
#include <fl/Fl_Table_Data.h>



/**
*  BaseView constructor
*/
ReportView::ReportView (const char* workSpace, int type, BaseReport* report, int x, int y, int w, int h, const char* label) :
BaseView (workSpace, type, x, y, w, h, label) {
    aReport     = report;
    aTable      = new Fl_Table (XTrader::GetApp(), 0, 0, 0, w, 0);
    aEditLayout->addRight (aTable);

    createToolbar (STRING(SELECT2), STRING(DATE2), STRING(RELOAD), STRING(PREV), STRING(NEXT), STRING(SAVE2), STRING(CLOSE_), 0);
    createBrowser ();
    loadBrowserData1 ();
    loadBrowserData2 ();
    enableButtons ();
    aTable->event_handler (this);

    if (aReport) cbBrowser1 ();
}



/**
*  Destroy view
*/
ReportView::~ReportView () {
    delete aReport;
}



/**
*  Check if view can be closed
*/
bool ReportView::CanClose () {
    if (aTable->dirty() && fl_ask (STRING(ASK_CLOSE_DIRTY)) == 0)
        return false;
    return true;
}



/**
*  Callback for browser
*/
void ReportView::cbBrowser1 () {
    int progType = 0;
    aMem1 = aMem2 = 0;

    try {
        switch (aType) {
            case GUARD: {
                break;
            }

            case PORTFOLIO: {
                int index = aBrowser1->value() - 1;
                if (CanClose() == false) return;
                if (!aList1 || index >= aList1->Size() || index < 0) return;
                Account* a = (Account*) aList1->Get(index);
                if (!aReport) aReport = new PortfolioReport (new Report());
                aReport->SetData (new Account (*a));
                progType = 1;
                aMem1 = a;
                break;
            }

            case SET: {
                int index = aBrowser1->value() - 1;
                if (CanClose() == false) return;
                if (!aList1 || index >= aList1->Size() || index < 0) return;
                if (!aReport) aReport = new PriceReport (new Report());
                aReport->SetData (new Set (*(Set*) (*aList1)[index]));
                aMem1 = ((Set*) (*aList1)[index]);
                break;
            }
        }

        if (aReport && aReport->GetReportType() >= 0) {
            XTrader::StartWork (STRING(INFO_CREATE_REPORT), progType);
            aReport->Create ();
            XTrader::StopWork ("");
            aTable->data (new ReportData (aReport->GetReport()));
            aTable->move_cursor (-9999999, -999999);
            redraw();
            Fl::check ();
        }
    }
    catch (const char* e) {
        XTrader::StopWork ("");
        fl_beep (FL_BEEP_ERROR);
        fl_alert (e);
    }

    enableButtons ();
    Fl::focus (aTable);
    Fl::redraw ();
}



/**
*  Callback for browser
*/
void ReportView::cbBrowser2 () {
}



/**
*  Callback for buttons
*/
void ReportView::cbButton (Fl_Button* button) {
    try {
        if (strcmp (button->label(), STRING(CLOSE_)) == 0)
            XTrader::Close (this);
        else if (strcmp (button->label(), STRING(DATE2)) == 0) {
            const char* start = (aReport->ChangeDate() == BaseReport::START_DATE || aReport->ChangeDate() == BaseReport::BOTH_DATE) ? aReport->GetStartDate()->Get() : 0;
            const char* stop  = (aReport->ChangeDate() == BaseReport::STOP_DATE || aReport->ChangeDate() == BaseReport::BOTH_DATE) ? aReport->GetStopDate()->Get() : 0;
            DateDlg     dlg (start, stop);

            if (dlg.show_modal (XTrader::GetApp()) == 1) {
                if (start) aReport->SetStartDate (dlg.StartDate());
                if (stop) aReport->SetStopDate (dlg.StopDate());
                cbBrowser1 ();
            }
        }
        else if (strcmp (button->label(), STRING(NEXT)) == 0) {
            aBrowser1->value (aBrowser1->value() + 1);
            cbBrowser1 ();
        }
        else if (strcmp (button->label(), STRING(PREV)) == 0) {
            aBrowser1->value (aBrowser1->value() - 1);
            cbBrowser1 ();
        }
        else if (strcmp (button->label(), STRING(RELOAD)) == 0)
            cbBrowser1 ();
        else if (strcmp (button->label(), STRING(SAVE2)) == 0) {
            Report* r = ((ReportData*)aTable->data())->report();

            if (r) {
                char* file = fl_file_chooser (STRING(DLG_SELECT_WRITE_FILE), "*.*", "");
                if (file && *file) r->Save (file);
            }
        }
        else if (strcmp (button->label(), STRING(SELECT2)) == 0) {
            SelectDlg dlg (STRING(DLG_SELECT_REPORT), aReport->Choice());

            if (dlg.show_modal (XTrader::GetApp()) == 1) {
                aReport->SetReportType (-dlg.Selection());
                cbBrowser1 ();
            }
        }
    }
    catch (const char* x) {
        fl_alert (x);
    }

    enableButtons ();
}



/**
*  Enable/disable buttons
*/
void ReportView::enableButtons () {
    disableButtons ();

    if (aTable->data())
        aTable->data()->border(false);
    if (aReport) {
        if (aReport) enableButton (STRING(RELOAD));
        if (aReport) enableButton (STRING(SAVE2));
        enableButton (STRING(SELECT2));
        if (aReport && aReport->ChangeDate() != BaseReport::NO_DATE) enableButton (STRING(DATE2));
        if (aBrowser1) {
            if (aBrowser1->value () > 0 && aBrowser1->value () < aList1->Size())  enableButton (STRING(NEXT));
            if (aBrowser1->value () > 1  && aBrowser1->value () <= aList1->Size())  enableButton (STRING(PREV));
        }
    }

    enableButton (STRING(CLOSE_));

    aToolbar->redraw ();
    Fl::focus (aTable);
}



/**
*
*/
int ReportView::handle (int event) {
    return Fl_Group::handle (event);
}
