/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "gui/XTrader.h"
#include "db/DBFactory.h"
#include "db/Import.h"
#include "db/Pref.h"
#include "dlg/AboutDlg.h"
#include "dlg/DownloadDlg.h"
#include "dlg/ExportDlg.h"
#include "dlg/InfoDlg.h"
#include "dlg/ProgressDlg.h"
#include "dlg/SetupDlg.h"
#include "dlg/SplitDlg.h"
#include "dlg/WorkSpaceDlg.h"
#include "Enums.h"
#include "gui/ChartView.h"
#include "gui/EditView.h"
#include "gui/ReportView.h"
#include "help/License.h"
#include "Progress.h"
#include "report/GuardReport.h"
#include "report/PortfolioReport.h"
#include "report/PriceReport.h"
#include "Resource.h"
#include "table/AccountData.h"
#include "table/GuardData.h"
#include "table/PriceData.h"
#include "table/ReportData.h"
#include "table/SecurityData.h"
#include "table/SetData.h"
#include "table/SetItemData.h"
#include <FL/Fl.h>
#include <FL/Fl_Box.h>
#include <FL/fl_file_chooser.h>
#include <fl/Fl_Layout.h>
#include <FL/Fl_Menu_Bar.h>
#include <fl/Fl_Table_Data.h>
#include <MHDate.h>
#include <MHDebug.h>
#include <MHFile.h>
#include <MHMessage.h>

#ifdef WIN32
    extern HINSTANCE fl_display;
#else
    #include <FL/x.H>
    #include <X11/xpm.h>
    #include <xtrader.xpm>
#endif

        XTrader*     XTrader::aaApp     = 0;
static  ProgressDlg* aaProgress         = 0;



/**
*  Update progressbar dialog during heavy work load
*/
void xtrader_update () {
    aaProgress->Value (xtrader_done(), "");
}



/**
*  Create application and main objects as menu, toolbar...
*  @param int  - Width
*  @param int  - Height
*/
XTrader::XTrader (int w, int h) :
Fl_Double_Window (w, h, STRING(APP_NAME)) {

    switch (DBFactory::GetPrefIntValue (STRING(PREF_THEME), 0, XTRADER_LAST_THEME - 1, 0)) {
        case 1:
            Fl::set_boxtype (FL_UP_BOX, FL_THIN_UP_BOX);
            Fl::set_boxtype (FL_UP_FRAME, FL_THIN_UP_FRAME);
            Fl::set_boxtype (FL_DOWN_FRAME, FL_THIN_DOWN_FRAME);
            Fl::set_boxtype (FL_DOWN_BOX, FL_THIN_DOWN_BOX);
            break;

        default:
            Fl::scheme ("plastic");
            break;
    }

    aLayout         = new Fl_BorderLayout (0, 0, 0, 0);
    aMenu           = new Fl_Menu_Bar (0, 0, 0, 25);
    aTop            = new Fl_BorderLayout (0, 0, 0, 27);
    aToolbar        = new Fl_HorizontalLayout (0, 0, 0, 0);
    aTabs           = new Tabs (0, 0, 0, 0);
    aStatus         = new Fl_Box (0, 0, 0, 20);
    aListeners      = new MHMessage();
    aLastClosedView = 0;
    aaApp           = this;
    aInitSize       = false;
    aX = aY = aW = aH = 0;
    aStatus->align (FL_ALIGN_INSIDE | FL_ALIGN_LEFT);
    aStatus->box (FL_UP_BOX);
    aLayout->space (5, 0, 0, 0);
    aTop->space (0, 0, 0, 0);
    aToolbar->space (2, 2);
    aToolbar->box (FL_UP_BOX);

    aMenu->textsize (XTrader::FontSize ());
    aTabs->labelsize (XTrader::FontSize ());
    aStatus->labelsize (XTrader::FontSize ());
    aStatus->labelfont(FL_COURIER);

    aTop->add (Fl_Layout::NORTH, aMenu);
    aTop->add (Fl_Layout::CENTER, aToolbar);
    aLayout->add (Fl_Layout::NORTH, aTop);
    aLayout->add (Fl_Layout::CENTER, aTabs);
    aLayout->add (Fl_Layout::SOUTH, aStatus);
    add (aLayout);

    size_range (480, 360, 0, 0, 0, 0, 0);
    resizable (this);
    user_data (this);

    createMenu ();
    callback ((Fl_Callback*)XTrader::cbClose_);

    BuildToolbar ();
}



/**
*  Delete application.
*  Delete last closed view also, which are owned by nobody
*/
XTrader::~XTrader (){
    delete aListeners;
    delete aLastClosedView;
}



/**
*  Create application toolbar
*/
void XTrader::BuildToolbar () {
    MHVector    ve;
    MHString    tool (DBFactory::GetPrefStringValue (STRING(PREF_TOOLBAR), ""));
    int         buttons = 0;

    aTop->resize (0, 0, 0, 26);
    aToolbar->resize (0, 0, 0, 0);
    aToolbar->clear ();

    tool.Split (':', &ve);

    for (int f = 0; f < ve.Size(); f++) {
        MHString* s = (MHString*) ve[f];

        if (s && s->Size() > 2) {
            if (buttons == 0) {
                aTop->resize (0, 0, 0, 55);
                aToolbar->resize (0, 0, 0, 29);
                buttons = 1;
            }

            int        cbId  = 0;
            Fl_Button* b     = 0;

            if (*s == STRING(TOOL_DATABASE)) {
                cbId = Resource::MENU_SELECT_DATABASE;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_DATABASE));
            }
            else if (*s == STRING(TOOL_DOWNLOAD)) {
                cbId = Resource::MENU_DOWNLOAD;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_DOWNLOAD));
            }
            else if (*s == STRING(TOOL_EXPORT)) {
                cbId = Resource::MENU_EXPORT;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_EXPORT));
            }
            else if (*s == STRING(TOOL_IMPORT)) {
                cbId = Resource::MENU_IMPORT;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_IMPORT));
            }
            else if (*s == STRING(TOOL_PREFERENCE)) {
                cbId = Resource::MENU_SETTINGS;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_PREFERENCE));
            }
            else if (*s == STRING(TOOL_PORTFOLIO_CHART)) {
                cbId = Resource::MENU_PORTFOLIO_CHART;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_PORTFOLIO_CHART));
            }
            else if (*s == STRING(TOOL_PORTFOLIO_REPORT)) {
                cbId = Resource::MENU_PORTFOLIO_REPORT;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_PORTFOLIO_REPORT));
            }
            else if (*s == STRING(TOOL_PRICE_CHART)) {
                cbId = Resource::MENU_PRICE_CHART;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_PRICE_CHART));
            }
            else if (*s == STRING(TOOL_PRICE_REPORT)) {
                cbId = Resource::MENU_GUARD_EDIT;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_PRICE_REPORT));
            }
            else if (*s == STRING(TOOL_PRICE_WATCH)) {
                cbId = Resource::MENU_GUARD_EDIT;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_PRICE_WATCH));
            }
            else if (*s == STRING(TOOL_PRICE_WATCH_REPORT)) {
                cbId = Resource::MENU_GUARD_REPORT;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_PRICE_WATCH_REPORT));
            }
            else if (*s == STRING(TOOL_QUIT)) {
                cbId = Resource::MENU_QUIT;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_QUIT));
            }
            else if (*s == STRING(TOOL_VIEW_LOG)) {
                cbId = Resource::MENU_LOG;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_VIEW_LOG));
            }
            else if (*s == STRING(TOOL_WORKSPACE)) {
                cbId = Resource::MENU_WORKSPACE;
                b    = new Fl_Button (0, 0, XTrader::FontSize() * 9, 0, STRING(TOOL_WORKSPACE));
            }

            if (cbId && b) {
                XTrader::FixButton (b);
                b->callback ((Fl_Callback*) XTrader::cbMenu_, (void*)cbId);
                aToolbar->add (b);
            }
        }
    }

    ve.Erase ();
    resize (x(), y(), w(), h());
    Fl::redraw ();
}



/**
*
*/
void XTrader::cbClose () {
    if (Close() == true)
        hide ();
}



/**
*  Callback for all menu items
*/
void XTrader::cbMenu (int id) {
    try {
        switch (id) {
            case Resource::MENU_ABOUT: {
                AboutDlg dlg;
                dlg.show_modal (this);
                break;
            }

            case Resource::MENU_ACCOUNT_EDIT:
                createView ("EditAccount");
                break;

            case Resource::MENU_CLEAR_CACHE:
                DBFactory::Clear ();
                break;

            case Resource::MENU_DOWNLOAD: {
                DownloadDlg downloadDlg;
                downloadDlg.show_modal (this);
                break;
            }

            case Resource::MENU_EXPORT: {
                ExportDlg exportDlg;
                exportDlg.show_modal (this);
                break;
            }

            case Resource::MENU_GUARD_EDIT:
                createView ("EditGuard");
                break;

            case Resource::MENU_GUARD_REPORT:
                createView ("ReportGuard");
                break;

            case Resource::MENU_IMPORT: {
                char* file = fl_file_chooser (STRING(DLG_IMPORT), STRING(DLG_IMPORT_FILTER), "");
                if (file && *file) {
                    MHDate date;
                    Import* import = 0;
                    double start = MHUtil::Time();

                    XTrader::StartWork (STRING(INFO_IMPORTING));
                    try {
                        import = new Import (file);
                        import->Read ();
                        char buffer[100];
                        snprintf (buffer, 99, "%s (%d %s / %.1fs) (%s=%d)", STRING(INFO_IMPORT_FINISHED), import->Rows(), STRING(ROWS), MHUtil::Time() - start, STRING(ERROR_), import->Errors());
                        XTrader::SetMessage (buffer);
                    }
                    catch (const char* message) {
                        XTrader::SetError (message);
                    }
                    XTrader::StopWork ("");
                    delete import;
                    aListeners->Send (0, "Import", "changed", 0, 0);
                    Fl::redraw ();
                }
                break;
            }

            case Resource::MENU_LICENSE: {
                char buffer[5000];
                strcpy (buffer, aaLicense);
                strcat (buffer, aaLicense2);
                strcat (buffer, aaLicense3);
                InfoDlg dlg (STRING(LICENSE), buffer);
                dlg.show_modal (this);
                break;
            }

            case Resource::MENU_LOG: {
                MHString name = DBFactory::Database();
                name += "xtrader.log";
                int size   = MHFile::Size (name());
                char* buff = new char[size + 5];
                FILE* file = fopen (name(), "rb");
                buff[0] = '\0';
                if (file) fread (buff, sizeof(char), size, file);
                if (file) fclose (file);
                InfoDlg dlg (STRING(LOG), buff);
                dlg.show_modal (this);
                delete []buff;
                break;
            }

            case Resource::MENU_PORTFOLIO_CHART:
                createView ("ChartPortfolio");
                break;

            case Resource::MENU_PORTFOLIO_EDIT:
                createView ("EditPortfolio");
                break;

            case Resource::MENU_PORTFOLIO_REPORT:
                createView ("ReportPortfolio");
                break;

            case Resource::MENU_PRICE_CHART:
                createView ("ChartPrice");
                break;

            case Resource::MENU_PRICE_EDIT:
                createView ("EditPrice");
                break;

            case Resource::MENU_PRICE_REPORT:
                createView ("ReportPrice");
                break;

            case Resource::MENU_QUIT:
                cbClose ();
                break;

            case Resource::MENU_README: {
                //InfoDlg dlg (STRING(README), aaReadme);
                //dlg.Show (this);
                break;
            }

            case Resource::MENU_SECURITY_EDIT:
                createView ("EditSecurity");
                break;

            case Resource::MENU_SELECT_DATABASE: {
                char* dir = fl_dir_chooser (STRING(DLG_SELECT_DB), "", 0);
                if (dir && *dir && Close() == true) {
                    DBFactory::Stop ();
                    DBFactory::Start (dir);
                    LoadWorkSpace ();
                    XTrader::SetMessage (STRING(INFO_NEW_DATABASE));
                }
                else
                    XTrader::SetError (STRING(INFO_CLOSE_ALL));
                break;
            }

            case Resource::MENU_SET_EDIT:
                createView ("EditSet");
                break;

            case Resource::MENU_SET_ITEMS_EDIT:
                createView ("EditSetItems");
                break;

            case Resource::MENU_SETTINGS: {
                SetupDlg dlg;
                dlg.show_modal (this);
                break;
            }

            case Resource::MENU_SPLIT: {
                SplitDlg splitDlg;
                splitDlg.show_modal (this);
                break;
            }

            case Resource::MENU_WORKSPACE: {
                WorkSpaceDlg dlg;
                dlg.show_modal (this);
                break;
            }

            case Resource::MENU_WRITEPROTECT: {
                Fl_Menu_Item* i = getMenuItem (STRING(MENU_WRITEPROTECT2));

                if (i) {
                    DBFactory::SetPrefIntValue (STRING(PREF_WRITEPROTECTED), 0, 1000, 0, i->value());
                    SavePos ();
                    SaveWorkSpace ();
                    Pref::Save (DBFactory::LoadPref ());
                }
                break;
            }

            default:
                break;
        }
    }
    catch (const char* message) {
        XTrader::SetError (message);
    }
    redraw ();
}



/**
*  Close one view but check for unsaved data first
*/
void XTrader::close (BaseView* view) {
    for (int f = 0; f < aTabs->children(); f++) {
        BaseView* w = (BaseView*) aTabs->child (f);

        if (w == view) {
            if (w->CanClose ()) {
                aTabs->remove (w);
                aListeners->RemoveListener (w);
                delete aLastClosedView;
                aLastClosedView = w;
            }
            break;
        }
    }
    redraw ();
}



/**
*  Close all views but if data has changed in view ask user
*/
bool XTrader::Close () {
    if (DBFactory::GetPrefIntValue (STRING(PREF_WRITEPROTECTED), 0, 1000, 0) == 0) {
       SavePos();
       SaveWorkSpace();
       Pref::Save (DBFactory::LoadPref ());
    }

    while (aTabs->children() > 0) {
        BaseView* w = (BaseView*) aTabs->child (0);

        if (w->CanClose ()) {
            aTabs->remove (w);
            aListeners->RemoveListener (w);
            delete aLastClosedView;
            aLastClosedView = w;
        }
        else {
            aTabs->value (w);
            redraw ();
            return false;
        }
    }

    return true;
}



/**
*  Create application menu
*/
void XTrader::createMenu () {
    aMenu->add (STRING(MENU_IMPORT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_IMPORT);
    aMenu->add (STRING(MENU_EXPORT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_EXPORT);
    aMenu->add (STRING(MENU_DOWNLOAD), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_DOWNLOAD, FL_MENU_DIVIDER);
    //aMenu->add (STRING(MENU_SPLIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_SPLIT, FL_MENU_DIVIDER);
    aMenu->add (STRING(MENU_LOG), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_LOG, FL_MENU_DIVIDER);
    //aMenu->add (STRING(MENU_CLEAR_CACHE), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_CLEAR_CACHE, FL_MENU_DIVIDER);
    aMenu->add (STRING(MENU_QUIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_QUIT);

    aMenu->add (STRING(MENU_SETTINGS), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_SETTINGS);
    aMenu->add (STRING(MENU_SELECT_DATABASE), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_SELECT_DATABASE);
    aMenu->add (STRING(MENU_WORKSPACE), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_WORKSPACE);
    aMenu->add (STRING(MENU_WRITEPROTECT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_WRITEPROTECT, FL_MENU_TOGGLE);

    aMenu->add (STRING(MENU_SECURITY_EDIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_SECURITY_EDIT);
    aMenu->add (STRING(MENU_SET_EDIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_SET_EDIT);
    aMenu->add (STRING(MENU_SET_ITEMS_EDIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_SET_ITEMS_EDIT);
    aMenu->add (STRING(MENU_PRICE_EDIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_PRICE_EDIT);
    aMenu->add (STRING(MENU_GUARD_EDIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_GUARD_EDIT, FL_MENU_DIVIDER);
    aMenu->add (STRING(MENU_PRICE_REPORT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_PRICE_REPORT);
    aMenu->add (STRING(MENU_GUARD_REPORT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_GUARD_REPORT, FL_MENU_DIVIDER);
    aMenu->add (STRING(MENU_PRICE_CHART), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_PRICE_CHART);

    aMenu->add (STRING(MENU_ACCOUNT_EDIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_ACCOUNT_EDIT);
    aMenu->add (STRING(MENU_PORTFOLIO_EDIT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_PORTFOLIO_EDIT, FL_MENU_DIVIDER);
    aMenu->add (STRING(MENU_PORTFOLIO_REPORT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_PORTFOLIO_REPORT, FL_MENU_DIVIDER);
    aMenu->add (STRING(MENU_PORTFOLIO_CHART), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_PORTFOLIO_CHART);

    aMenu->add (STRING(MENU_ABOUT), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_ABOUT);
    aMenu->add (STRING(MENU_LICENSE), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_LICENSE);
    //aMenu->add (STRING(MENU_README), 0, (Fl_Callback*) XTrader::cbMenu_, (void*)Resource::MENU_README);
}



/**
*  Create a view with perhaps some data.
*  @param const char* - View + data name
*  @param const char* - Data class name, if 0 then do data is created
*  @return BaseView*  - The created view or 0
*/
BaseView* XTrader::createView (const char* view) {
    BaseView* b = 0;

    if (strcmp (view, "ChartPortfolio") == 0)
        b = new ChartView ("ChartPortfolio", BaseView::PORTFOLIO, aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(PORTFOLIO_CHART_VIEW));
    else if (strcmp (view, "ChartPrice") == 0)
        b = new ChartView ("ChartPrice", BaseView::SET_SETITEM, aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(PRICE_CHART_VIEW));
    else if (strcmp (view, "EditAccount") == 0) {
        if (findView (STRING(ACCOUNT_EDIT_VIEW)) == true) return 0;
        b = new EditView ("EditAccount", BaseView::NO_BROWSER, new AccountData(), aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(ACCOUNT_EDIT_VIEW));
    }
    else if (strcmp (view, "EditGuard") == 0) {
        if (findView (STRING(GUARD_EDIT_VIEW)) == true) return 0;
        b = new EditView ("EditGuard", BaseView::NO_BROWSER, new GuardData(), aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(GUARD_EDIT_VIEW));
    }
    else if (strcmp (view, "EditPortfolio") == 0) {
        if (findView (STRING(PORTFOLIO_EDIT_VIEW)) == true) return 0;
        b = new EditView ("EditPortfolio", BaseView::PORTFOLIO, 0, aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(PORTFOLIO_EDIT_VIEW));
    }
    else if (strcmp (view, "EditSecurity") == 0) {
        if (findView (STRING(SECURITY_EDIT_VIEW)) == true) return 0;
        b = new EditView ("EditSecurity", BaseView::NO_BROWSER, new SecurityData(), aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(SECURITY_EDIT_VIEW));
    }
    else if (strcmp (view, "EditSet") == 0) {
        if (findView (STRING(SET_EDIT_VIEW)) == true) return 0;
        b = new EditView ("EditSet", BaseView::NO_BROWSER, new SetData(), aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(SET_EDIT_VIEW));
    }
    else if (strcmp (view, "EditSetItems") == 0) {
        if (findView (STRING(SET_ITEMS_EDIT_VIEW)) == true) return 0;
        b = new EditView ("EditSetItems", BaseView::SET, 0, aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(SET_ITEMS_EDIT_VIEW));
    }
    else if (strcmp (view, "ReportGuard") == 0) {
        if (findView (STRING(GUARD_REPORT)) == true) return 0;
        b = new ReportView ("ReportGuard", BaseView::GUARD, new GuardReport (new Report()), aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(GUARD_REPORT));
    }
    else if (strcmp (view, "ReportPortfolio") == 0)
        b = new ReportView ("ReportPortfolio", BaseView::PORTFOLIO, 0, aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(PORTFOLIO_REPORT_VIEW));
    else if (strcmp (view, "EditPrice") == 0) {
        if (findView (STRING(PRICE_EDIT_VIEW)) == true) return 0;
        b = new EditView ("EditPrice", BaseView::SET_SETITEM, 0, aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(PRICE_EDIT_VIEW));
    }
    else if (strcmp (view, "ReportPrice") == 0)
        b = new ReportView ("ReportPrice", BaseView::SET, 0, aTabs->x(), aTabs->y() + 20, aTabs->w(), aTabs->h() - 20, STRING(PRICE_REPORT_VIEW));

    if (b) {
        aListeners->AddListener (b);
        aTabs->add (b);
        aTabs->value (b);
        /*aTabs->*/resize (x(), y(), w(), h());
        redraw ();
    }

    return b;
}



/**
*  Check if view exist already, if so return true
*/
bool XTrader::findView (const char* label) {
    for (int f = 0; f < aTabs->children(); f++) {
        BaseView* v = (BaseView*) aTabs->child (f);

        if (strcmp (label, v->label()) == 0) {
            aTabs->value (v);
            return true;
        }
    }
    return false;
}


/**
*  Change button appearance
*  @param Fl_Button* - Button
*/
void XTrader::FixButton (Fl_Button* b) {
    switch (DBFactory::GetPrefIntValue (STRING(PREF_THEME), 0, XTRADER_LAST_THEME - 1, 0)) {
        case 0: {
            int col = DBFactory::GetPrefIntValue (STRING(PREF_BCOLOR), 0, 255, 0);
            switch (col) {
                case 0:
                    break;

                default:
                    b->color (col);
                    break;
            }
            break;
        }
    }

    b->labelsize (XTrader::FontSize ());
}



/**
*  Return the global font size
*  @return int  - Font size
*/
int XTrader::FontSize () {
    switch (DBFactory::GetPrefIntValue (STRING(PREF_FONTSIZE), 0, XTRADER_LAST_FONT, XTRADER_FONT12)) {
        case 0:
            return 10;

        case 2:
            return 14;

        default:
            return 12;
    }
}



/**
*  Get menu item with help of menu text. Only the last part of a fltk menu string must be used (after the last "/").
*/
Fl_Menu_Item* XTrader::getMenuItem (const char* label) {
    Fl_Menu_Item* i = (Fl_Menu_Item*) aMenu->menu ();
    for (int f = 0; f < aMenu->size(); f++) {
        if (!i->text)
            ;
        else if (strcmp (i->text, label) == 0)
            return i;
        i++;
    }
    return 0;
}



/**
*  Window event
*/
int XTrader::handle (int event) {
    switch (event) {
        case FL_KEYDOWN:
            //printf ("0 - %d - %d\n", Fl::event_key(), Fl::event_state());
            BaseView* w = (BaseView*) aTabs->value ();
            if (w)
                return w->handle (event);
            break;
    }
    return Fl_Double_Window::handle (event);
}



/**
*  Load work spaces
*/
void XTrader::LoadWorkSpace () {
    MHString s = DBFactory::GetPrefStringValue (STRING(PREF_WORKSPACE));
    MHVector ve;

    aTabs->off (true);
    s.Split (',', &ve);
    //  For every view entry
    for (int f = 0; f < ve.Size(); f++) {
        MHString* s2 = (MHString*) ve[f];
        MHVector  ve2;

        s2->Split(':', &ve2);
        if (ve2.Size() > 0) {
            MHString* name = (MHString*) ve2.Get(0);
            BaseView* v    = createView (name->Get());

            if (v)
                v->LoadWorkSpace (s2->Get());
        }
        ve2.Erase ();
    }
    ve.Erase ();

    Fl_Menu_Item* item = getMenuItem (STRING(MENU_WRITEPROTECT2));

    if (item) {
        if (DBFactory::GetPrefIntValue (STRING(PREF_WRITEPROTECTED), 0, 1000, 0) == 0)
            item->clear ();
        else
            item->set ();
    }

    BuildToolbar ();
    aLabel = STRING(APP_NAME);
    aLabel += " - ";
    aLabel += DBFactory::GetWorkSpaceName ();
    aLabel += " ";
    aLabel += STRING(WORKSPACE);
    label (aLabel());
    aTabs->off (false);
}



/**
*  Overides Fl_Widget::resize to take care of it's own resizes
*  and also resize menu, toolbar, statusbar and the tile window
*  @param int  - X pos of application
*  @param int  - Y pos of application
*  @param int  - Width of application
*  @param int  - Height of application
*/
void XTrader::resize (int x, int y, int w, int h) {
    DBG ("XTrader::resize")

    DBG_LOG4 ("XTrader    %4d, %4d, %4d, %4d", x,y,w,h)

    aLayout->off (true);
    aTabs->off (true);

    Fl_Double_Window::resize (x, y, w, h);

    DBG_LOG4 ("XTrader    %4d, %4d, %4d, %4d", this->x(), this->y(), this->w(), this->h())

    aLayout->off (false);
    aLayout->resize (0, 0, w, h);
    aTabs->off (false);
    aTabs->resize (aTabs->x(), aTabs->y(), aTabs->w(), aTabs->h());

    //printf ("--------------\nX%3d, %4d   %4d, %4d\n", this->x(), this->y(), this->w(), this->h());
    DBG_LOG4 ("XTrader    %4d, %4d, %4d, %4d", this->x(), this->y(), this->w(), this->h())
    DBG_LOG4 ("aLayout    %4d, %4d, %4d, %4d", aLayout->x(),  aLayout->y(),  aLayout->w(),  aLayout->h())
    DBG_LOG4 ("aTop       %4d, %4d, %4d, %4d", aTop->x(),     aTop->y(),     aTop->w(),     aTop->h())
    DBG_LOG4 ("aMenu      %4d, %4d, %4d, %4d", aMenu->x(),    aMenu->y(),    aMenu->w(),    aMenu->h())
    DBG_LOG4 ("aToolbar   %4d, %4d, %4d, %4d", aToolbar->x(), aToolbar->y(), aToolbar->w(), aToolbar->h())
    DBG_LOG4 ("aTabs      %4d, %4d, %4d, %4d", aTabs->x(),    aTabs->y(),    aTabs->w(),    aTabs->h())
    DBG_LOG4 ("aStatus    %4d, %4d, %4d, %4d", aStatus->x(),  aStatus->y(),  aStatus->w(),  aStatus->h())
}



/**
*  Save position to database
*/
void XTrader::SavePos() {
    DBFactory::SetPrefIntValue (STRING(PREF_APPX), 0, 1000,  20, this->x());
    DBFactory::SetPrefIntValue (STRING(PREF_APPY), 0, 1000,  20, this->y());
    DBFactory::SetPrefIntValue (STRING(PREF_APPW), 0, 3000, 640, this->w());
    DBFactory::SetPrefIntValue (STRING(PREF_APPH), 0, 3000, 480, this->h());
}



/**
*  Save work spaces
*/
void XTrader::SaveWorkSpace() {
    MHString s;

    for (int f = 0; f < aTabs->children(); f++) {
        BaseView* v = (BaseView*) aTabs->child (f);
        if (f > 0) s += ",";
        s += v->GetWorkSpace ();
    }
    DBFactory::SetPrefStringValue (STRING(PREF_WORKSPACE), s());
}



/**
*  Show application
*/
void XTrader::Show(int argc, char** argv) {
    Fl::visual (FL_DOUBLE | FL_RGB);
    size (DBFactory::GetPrefIntValue (STRING(PREF_APPW), 0, 3000, 800), DBFactory::GetPrefIntValue (STRING(PREF_APPH), 0, 3000, 600));
    position (DBFactory::GetPrefIntValue (STRING(PREF_APPX), 0, 1000, 50), DBFactory::GetPrefIntValue (STRING(PREF_APPY), 0, 1000, 50));

    #ifdef WIN32
        HICON i = LoadIcon (fl_display, MAKEINTRESOURCE(666));
        icon ((char*) i);
    #else
        //fl_open_display ();
        Pixmap pixmap, mask;
        XpmAttributes attr;
        memset(&attr, 0, sizeof(attr));
        XpmCreatePixmapFromData(fl_display, DefaultRootWindow(fl_display), xtrader_xpm, &pixmap, &mask, &attr);
        icon((char*) pixmap);
    #endif

    show (argc, argv);
}



/**
*  Start some time consuming work
*/
void XTrader::StartWork (const char* message, int showtime) {
    DBG ("XTrader::StartWork")

    if (aaProgress)
        XTrader::StopWork ("");

    aaProgress = new ProgressDlg (message, 0, showtime);
    aaProgress->show_modal2 (XTrader::GetApp());
}



/**
*  Stop thread or reset cursor
*/
void XTrader::StopWork (const char* message) {
    DBG ("XTrader::StopWork")

    if (aaProgress) {
        MHUtil::Sleep (0.5);
        aaProgress->hide();
        delete aaProgress;
        aaProgress = 0;
        if (message && *message)
            XTrader::SetMessage (message);
    }
}
