/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <report/GuardReport.h>
#include <db/DBFactory.h>
#include <db/Guard.h>
#include <db/Price.h>
#include <db/Security.h>
#include <Resource.h>
#include <Progress.h>
#include <MHPrice.h>
#include <MHDate.h>



/**
*
*/
GuardReport::GuardReport (Report* report) :
BaseReport (report, GuardReport::GUARD) {
    aChoice.aNames.Push (new MHString (STRING(GUARD_REPORT)));
    aChoice.aTypes.Push (new MHString ((int)GUARD));
}



/**
*  Check a security object if security should be sold or bought
*  @param Guard* - Object to check
*  @return int     - BUY, SELL or HOLD
*/
int GuardReport::checkGuard (Guard* guard, MHVector* prices) {
    MHPrice p1(aStartDate(), 0, 0, 0, 0, 0);
    Price*  price = (Price*) prices->Find (&p1, MHVector::FIND_BEFORE, Price::SORT_DATE, 0);

    if (!price) return HOLD;
    guard->SetCurrDate (price->aDate);
    guard->SetCurrValue (price->aClose);

    switch (guard->GetType()) {
        case Guard::SELL_LOW:
            if (guard->GetValue() > price->aClose) return SELL;
            break;

        case Guard::BUY_LOW:
            if (guard->GetValue() > price->aClose) return BUY;
            break;

        case Guard::SELL_HIGH:
            if (guard->GetValue() < price->aClose) return SELL;
            break;

        case Guard::BUY_HIGH:
            if (guard->GetValue() < price->aClose) return BUY;
            break;

        default:
            break;
    }
    return HOLD;
}



/**
*  Create new guard report
*  @param StringVect& - Vector with report strings
*/
void GuardReport::Create () {
    aReport->Erase ();

    switch (aReportType) {
    case GUARD:
        createGuardReport (false);
        break;

    case GUARD_SHORT:
        createGuardReport (true);
        break;
    }
}



/**
*  Create a price list, wide version.
*  @param MHVector*  - Vector with report strings
*/
void GuardReport::createGuardReport (bool shortReport) {
    int           f, status;
    MHVector*     guardItems;
    MHVector*     prices;
    MHVector*     row;
    Guard*        guard;
    Security*     security;

    createGuardReportHeader (shortReport);

    //  Load set and prices and create one row for every security
    guardItems = DBFactory::LoadGuard ();

    xtrader_reset (guardItems->Size());
    guardItems->Sort();

    for (f = 0; f < guardItems->Size(); f++) {
        guard    = (Guard*) (*guardItems)[f];
        security = guard->LoadSecurity();
        prices   = DBFactory::LoadPrice (security);
        status   = checkGuard (guard, prices);

        if (status == SELL)
            row = createGuardReportRow (shortReport, STRING(SELL), guard);
        else if (status == BUY)
            row = createGuardReportRow (shortReport, STRING(BUY), guard);
        else
            row = createGuardReportRow (shortReport, STRING(HOLD), guard);

        aReport->aRows.Push (row);
        xtrader_add(1);
        if (xtrader_abort()) break;
    }
}



/**
*  Create a report header
*  @param bool - true to use short version
*/
void GuardReport::createGuardReportHeader (bool shortReport) {
    if (shortReport == false) {
        aReport->aTitle = new MHString (100, "%s %s", STRING(GUARD_REPORT), aStartDate());
        aReport->aHeader.Push (new MHString (30, "%s", STRING(NAME)));
        aReport->aHeader.Push (new MHString (30, "%s", STRING(EVENT)));
        aReport->aHeader.Push (new MHString (30, "%s", STRING(LIMIT)));
        aReport->aHeader.Push (new MHString (30, "%s", STRING(DATE)));
        aReport->aHeader.Push (new MHString (30, "%s", STRING(LAST)));

        aReport->aHeader.Push (new MHString (30, "%s", STRING(STATUS)));
        aReport->aAlign.Push (new MHString (5, "%c", Report::LEFT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::LEFT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    }
    else
        aReport->aHeader.Push (new MHString (30, "%s", aStartDate()));
}



/**
*  Create a report header
*  @param bool - true to use short version
*/
MHVector* GuardReport::createGuardReportRow (bool shortReport, const char* signal, Guard* guard) {
    MHVector* row = new MHVector ();

    guard->LoadSecurity ();

    if (shortReport == false) {
        if (guard->GetSecurity())
            row->Push (new MHString (30, "%s", guard->GetSecurity()->GetName()->Get()));
            row->Push (new MHString (30, "%s", guard->GetTypeName()));
            row->Push (new MHString (30, "%.2f", guard->GetValue()));
            row->Push (new MHString (20, "%s", guard->GetCurrDate()->Get()));
            row->Push (new MHString (30, "%.2f", guard->GetCurrValue()));
            row->Push (new MHString (30, "%s", signal));
    }
    else {
    }


    return row;
}


/**
*  Check is the message is for me and create report again if so
*/
void GuardReport::Reload (const char* message) {
    if (*message == '\0' || strcmp (message, "Guard") == 0 || strcmp (message, "Security") == 0) {
        DBFactory::ClearGuard ();
        Create();
    }
}

