/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "report/PortfolioValue.h"
#include "db/Portfolio2.h"
#include <MHDebug.h>



/**
*  Create new object with all data set to 0.
*/
PortfolioValue::PortfolioValue () {
    aBuyValue      = 0.0;
    aBuyValue2     = 0.0;
    aCurrCash      = 0.0;
    aCurrValue     = 0.0;
    aDivValue      = 0.0;
    aNetCash       = 0.0;
    aSharePrice    = 0.0;
    aShareVolume   = 0.0;
}



/**
*  Add cash to portfolio value
*  @param Portfolio2* - Transaction with money
*  @param MHVector*   - List with buy/sell/dividend trades
*/
void PortfolioValue::AddCash (Portfolio2* p2, MHVector* in) {
    if (aSharePrice < 0.000001) {
        //  Money is transfered into the account for the first time
        aSharePrice   = 100.0;
        aShareVolume += (p2->Value() / aSharePrice);
        aNetCash     += p2->Value();
    }
    else {
        //  Money is added to account, calc new number of shares
        if (in)
            Recalc (p2->GetDate()->Get(), in);
        aSharePrice   = (aCurrValue + aCurrCash - p2->Value()) / aShareVolume;
        aShareVolume += (p2->Value() / aSharePrice);
        aNetCash     += p2->Value();
    }
}



/**
*  Log to debug
*/
void PortfolioValue::Log () {
    DBG ("PortfolioValue::Log");
    DBG_LOG1 ("aBuyValue    =%f", aBuyValue)
    DBG_LOG1 ("aBuyValue2   =%f", aBuyValue2)
    DBG_LOG1 ("aCurrValue   =%f", aCurrValue)
    DBG_LOG1 ("aDivValue    =%f", aDivValue)
    DBG_LOG1 ("aCurrCash    =%f", aCurrCash)
    DBG_LOG1 ("aNetCash     =%f", aNetCash)
    DBG_LOG1 ("aSharePrice  =%f", aSharePrice)
    DBG_LOG1 ("aShareVolume =%f", aShareVolume)
}



/**
*  Recalc portfolio value
*  @param const char*   - Current date
*  @param MHVector*     - List with buy/sell/dividend trades
*/
void PortfolioValue::Recalc (const char* date, MHVector* in) {
    DBG ("PortfolioValue::Recalc");
    DBG_LOG2 ("date=%s, insize=%d", date, in->Size());

    MHVector group;

    Portfolio2::GroupTrades (in, &group);
    Portfolio2::LoadCurrPrices (date, &group);
    Value (&group);
    group.Erase ();
}



/**
*  Remove cash from portfolio value
*  @param Portfolio2* - Transaction with money
*  @param MHVector*   - List with buy/sell/dividend trades
*/
void PortfolioValue::RemoveCash (Portfolio2* p2, MHVector* in) {
    //  Calc new share value, money is going out from the the account
    if (aCurrValue < 0.000001) {
        aSharePrice   = 100;
        aShareVolume -= (p2->Value() / aSharePrice);
    }
    else {
        Recalc (p2->GetDate()->Get(), in);
        aSharePrice = (aCurrValue + aCurrCash + p2->Value()) / aShareVolume;
        aShareVolume -= (p2->Value() / aSharePrice);
    }
}



/**
*  Force portfolio value to 100.
*  @param const char*   Start date
*  @param MHVector*     List with transactions
*/
void PortfolioValue::Reset (const char* startdate, MHVector* buyList) {
    DBG ("PortfolioValue::Reset")

    if (buyList)
        Recalc (startdate, buyList);
    aSharePrice    = 100.0;
    aNetCash       = aCurrCash + aCurrValue;
    aShareVolume   = (aCurrCash + aCurrValue) / 100.0;
    Log ();
}



/**
*  Set some portfolio values for a list of transactions.
*  @param MHVector*  - The vector with transactions
*/
void PortfolioValue::Value (MHVector* in) {
    aDivValue = aBuyValue = aBuyValue2 = aCurrValue = 0;

    for (int f = 0; f < in->Size(); f++) {
        Portfolio2* p2 = (Portfolio2*) (*in)[f];

        if (p2->GetSecurityID() > 0) {
            aBuyValue   += p2->BuyValue ();
            aBuyValue2  += p2->BuyValue2 ();
            aCurrValue  += p2->Value ();
            aDivValue   += p2->DivValue ();
        }
    }
}
