/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/PortfolioData.h>
#include <db/Account.h>
#include <db/DBFactory.h>
#include <db/Portfolio.h>
#include <db/Portfolio2.h>
#include <db/Security.h>
#include <gui/XTrader.h>
#include <Resource.h>
#include <MHDate.h>
#include <MHString.h>
#include <FL/Fl.h>



/**
*  @param Security* - The security object to load price data for. Caller is responsible for deleting object
*/
PortfolioData::PortfolioData (Account* a) :
BaseData () {
    aAccount        = *a;
    aVector           = new MHVector (500);
    aVectorSecurity = new MHVector (500);
    aVectorTotal    = new MHVector (500);
    aVectorBalance  = new MHVector (500);
    aChoiceEvent    = MHUtil::Allocate (Portfolio::EV_TRANSFER, 31);

    Portfolio::Load (&aAccount, aVector);

    for (int f = 0; f < Portfolio::EV_TRANSFER; f++)
        STRNCPY (aChoiceEvent[f], Portfolio::GetEventName(f), 30)
    reload ("Security");
    calcBalance ();
}



/**
*
*/
PortfolioData::~PortfolioData () {
    MHUtil::Delete (aChoiceEvent, Portfolio::EV_TRANSFER);
    if (aChoiceSecurity)
        MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);
    aVector->Erase ();
    aVectorSecurity->Erase ();
    aVectorTotal->Erase ();
    aVectorBalance->Erase ();
    delete aVector;
    delete aVectorSecurity;
    delete aVectorTotal;
    delete aVectorBalance;
}



/**
*
*/
void PortfolioData::add () {
    insert (aVector->Size());
}



/**
*
*/
Fl_Align PortfolioData::align (int row, int col) {
    switch (col) {
        case 4:
        case 5:
        case 6:
        case 7:
        case 8:
            return FL_ALIGN_RIGHT;

        case 0:
        case 1:
        case 2:
        case 3:
        default:
            return FL_ALIGN_LEFT;
    }
}



/**
*  Calc. transaction balance
*/
void PortfolioData::calcBalance () {
    double balance = aAccount.GetBalance();

    aVectorTotal->Erase ();
    aVectorBalance->Erase ();

    for (int f = 0; f < aVector->Size(); f++) {
        Portfolio2* p2 = (Portfolio2*) (*aVector)[f];

        balance += p2->CashValue();
        aVectorTotal->Push (new MHString (p2->CashValue()));
        aVectorBalance->Push (new MHString (balance));
    }
}



/**
*
*/
const char** PortfolioData::choice (int row, int col, int& size, int& selected, char* label) {
    int         f;
    Portfolio*  p = (Portfolio*) aVector->Get (row);

    switch (col) {
        case 1:
            //  Portfolio events
            size     = Portfolio::EV_TRANSFER;
            selected = 0;
            for (f = 0; f < Portfolio::EV_TRANSFER; f++) {
                if (strcmp (Portfolio::GetEventName(p->GetEvent()), aChoiceEvent[f]) == 0) {
                    selected = f;
                    break;
                }
            }
              return (char const**) aChoiceEvent;

        case 3:
            //  Security for portfolio buy/sell/dividend event
            size     = aVectorSecurity->Size() + 1;
            selected = 0;

            for (f = 0; f < aVectorSecurity->Size(); f++) {
                Security* s = (Security*) (*aVectorSecurity)[f];
                if (s->GetID() == p->GetSecurityID()) {
                    selected = f + 1;
                    break;
                }
            }
            strncpy (label, STRING(SELECT_SECURITY), 99);
            return (char const**) aChoiceSecurity;
    }

    return 0;
}



/**
*
*/
FL_TABLE_EDITOR PortfolioData::editor_type (int row, int col, bool force_custom) {
    switch (col) {
        case 0:
        case 2:
            return FL_TEXT_EDITOR;

        case 1:
            return FL_LIST_EDITOR;

        case 3:
            return FL_DLG_LIST_EDITOR;

        case 7:
        case 8:
            return FL_NO_EDITOR;

        default:
            return FL_FLOAT_EDITOR;
    }
}



/**
*
*/
void PortfolioData::erase (int row) {
    if (row >= 0 && row < aVector->Size()) {
        aVector->Erase (row);
        dirty (true);
    }
}



/**
*
*/
void PortfolioData::insert (int row) {
    MHDate     d;
    Portfolio* p = new Portfolio (d.Get (MHDate::LONG_DATE), Portfolio::EV_IN, 0, 0.0, 0.0, 0.0, 0, 0, "");

    aVector->Insert (p, row);
    aVectorTotal->Insert (new MHString(), row);
    aVectorBalance->Insert (new MHString(), row);
    dirty (true);
}



/**
*
*/
void PortfolioData::reload (const char* message) {
    if (strcmp (message, "Security") == 0) {
        Portfolio::LoadSecurities (aVector);

        if (aVectorSecurity->Size() > 0) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);
        aVectorSecurity->Erase ();
        Security::Load (aVectorSecurity);
        aVectorSecurity->Sort (Security::SORT_NAME);

        aChoiceSecurity = MHUtil::Allocate (aVectorSecurity->Size() + 1, 31);
        STRNCPY (aChoiceSecurity[0], " ", 30)
        for (int f = 1; f <= aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f - 1];
            STRNCPY (aChoiceSecurity[f], s->GetName()->Get(), 30)
        }
    }
    else if (strcmp (message, "Account") == 0) {
        calcBalance ();
    }
}



/**
*
*/
int PortfolioData::save () {
    Portfolio::Save (&aAccount, aVector);
    calcBalance ();
    dirty (false);
    XTrader::SetInfo (STRING(DATA_SAVED));
    XTrader::SendMessage (0, "Portfolio", "changed", "", &aAccount);
    return -1;
}



/**
*
*/
void PortfolioData::sort (int col, bool ascending) {
}



/**
*
*/
bool PortfolioData::value (const char* data, int row, int col) {
    Portfolio* p  = (Portfolio*) (*aVector)[row];

    switch (col) {
        case 0:
            if (strcmp (p->GetDate()->Get(), data) == 0) return true;
            if (MHDate::ValidDate (data) == false)
                return false;
            p->SetDate (data);
            break;

        case 1: {
            for (int f = 0; f < Portfolio::EV_TRANSFER; f++) {
                if (strcmp (Portfolio::GetEventName(f), data) == 0) {
                    p->SetEvent ((Portfolio::EVENT)f);
                    break;
                }
            }
            break;
        }

        case 2:
            p->SetMemo (data);
            break;

        case 3:
            if (strcmp (data, "") == 0)
                p->SetSecurityID (0);
            else {
                for (int f = 0; f < aVectorSecurity->Size(); f++) {
                    Security* s = (Security*) (*aVectorSecurity)[f];

                    if (strcmp (s->GetName()->Get(), data) == 0) {
                        if (p->GetEvent() >= Portfolio::EV_USE_SECURITY)
                            p->SetSecurityID (s->GetID());
                        else
                            p->SetSecurityID (0);
                        break;
                    }
                }
            }
            p->LoadSecurity ();
            break;

        case 4:
            p->SetVolume (atof(data));
            break;

        case 5:
            p->SetPrice (atof(data));
            break;

        case 6:
            p->SetCourtage (atof(data));
            break;
    }

    if (p->GetEvent() < Portfolio::EV_USE_SECURITY) {
        p->SetVolume (0.0);
        p->SetCourtage (0.0);
        p->SetSecurityID (0);
        p->LoadSecurity ();
    }

    dirty (true);
    return true;
}



/**
*
*/
const char* PortfolioData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return aAccount.GetAccount()->Get();
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(DATE);

            case 1:
                return STRING(TYPE);

            case 2:
                return STRING(MEMO);

            case 3:
                return STRING(SECURITY);

            case 4:
                return STRING(VOLUME);

            case 5:
                return STRING(PRICE);

            case 6:
                return STRING(COURTAGE);

            case 7:
                return STRING(TOTAL);

            case 8:
                return STRING(BALANCE);

            default:
                return "";
        }
    }
    else {
        Portfolio* p = (Portfolio*) (*aVector)[row];

        switch (col) {
            case 0:
                return p->GetDate()->Get();

            case 1:
                return p->GetEventName(p->GetEvent());

            case 2:
                return p->GetMemo()->Get();

            case 3:
                return p->GetSecurity()->GetName()->Get();

            case 4:
                snprintf (aBuff, 30, "%.2f", p->GetVolume());
                return aBuff;

            case 5:
                snprintf (aBuff, 30, "%.2f", p->GetPrice());
                return aBuff;

            case 6:
                snprintf (aBuff, 30, "%.2f", p->GetCourtage());
                return aBuff;

            case 7:
                return ((MHString*) (*aVectorTotal)[row])->Get();

            case 8:
                return ((MHString*) (*aVectorBalance)[row])->Get();

            default:
                return "";
        }
    }
}



/**
*
*/
int PortfolioData::width (int col) {
    switch (col) {
        case 2:
        case 3:
            return XTrader::FontSize() * 8;

        default:
            return XTrader::FontSize() * 6;
    }
}
